#ifndef _2255VX_H
#define _2255VX_H

//vendor request direction 
#define DIR_IN				0
#define DIR_OUT				1 
/**  Vendor specific commands for the USB chip
 *  @author: Dean Anderson
 *  @company: Sensoray Company Inc.  
 *  @copyright: 2007 Sensoray Company Inc.
 */
// vendor requests
// 0--0x0c are reserved as are 0xa0+

// SETUPDAT[0] == request type
// SETUPDAT[1] == request
// SETUPDAT[2] == value Low  (these are reversed in the driver order)
// SETUPDAT[3] == value High
// SETUPDAT[4] == index low 
// SETUPDAT[5] == index high
// SETUPDAT[6] == length low
// SETUPDAT[7] == length high 

//Define vendor request value

/**  VX_RESET resets input endpoints
  * @param valueL: unused
  * @param valueH: unused
  * @param indexL: unused
  * @param indexH: unused
  * @param lengthL: unused
  * @param lengthH: unused 
  * @return undefined 
  */
#define VX_RESET	 	0x10            // buffer Reset command

/**  VX_WRITESINGLE writes addr/data to Stratix
  * @param valueL: Data value to write
  * @param valueH: unused
  * @param indexL: Addr to write to
  * @param indexH: unused
  * @param lengthL: unused
  * @param lengthH: unused 
  * @return EP0BUF[0] = VX_WRITESINGLE
  */
#define VX_WRITESINGLE  0x12            // Vendor Write command

/**  VX_WRITEBURST writes data to Stratix.  Send the address to write to in this
  *  command and then start writing data to EP8(the write endpoint)
  * @param valueL: unused
  * @param valueH: unused
  * @param indexL: Addr to write to
  * @param indexH: unused
  * @param lengthL: unused
  * @param lengthH: unused 
  * @return EP0BUF[0] = VX_WRITEBURST
  */
#define VX_WRITEBURST   0x13            // Vendor Write command

/**  VX_READ reads data tfrom Stratix.  if valueL > 0, sets up valueL block transfers(FIFOREADs).
  *  if valueL == 0, reads single Byte(SingleRead Waveform)
  * @param valueL: LSB number of block transfers to do( 0--single read)
  * @param valueH: MSB number of block transfers to do
  * @param indexL: Addr to read from
  * @param indexH: unused
  * @param lengthL: unused
  * @param lengthH: unused 
  * @return if valueL==0, EP0BUF[0] = data read, else EP0BUF[0] =VX_READ( blocktransfer)
  */
#define VX_READ			0x14            // Vendor Read command


/**  VX_I2CWRITE writes to the I2C.  Note: device address in Value field here, not index.
  * @param valueL: i2cDevice address
  * @param valueH: unused
  * @param indexL: unused
  * @param indexH: unused
  * @param lengthL: length of data to write( writes EP0 buffer length to I2C, include subaddress in data)
  * @param lengthH: unused 
  * @return EP0BUF[0] = VX_I2CWRITE
  */

#define VX_I2CWRITE		0x20              // I2C write

/**  VX_I2CREAD_NOSTART reads from the I2C.  Note: device address in Value field here, not index.
  * @param valueL: i2cDevice address
  * @param valueH: unused
  * @param indexL: unused
  * @param indexH: unused
  * @param lengthL: length of data to read
  * @param lengthH: unused 
  * @return EP0BUF[0..length] = I2C data read (all fields 0xcd if error occurs)
  */
#define VX_I2CREAD_NOSTART	0x21              // I2C read with no Start address


/**  VX_I2CREAD reads from the I2C.  Note: device address in Value field here, not index.
  * @param valueL: i2cDevice address
  * @param valueH: unused
  * @param indexL: i2c sub address index to read from
  * @param indexH: unused
  * @param lengthL: length of data to read
  * @param lengthH: unused 
  * @return EP0BUF[0..length] = I2C data read (all fields 0xcd if error occurs)
  */
#define VX_I2CREAD		0x22              // I2C read with Start address


/**  VX_FW reports I2c firmware.
  * @param valueL: unused
  * @param valueH: unused
  * @param indexL: unused
  * @param indexH: unused
  * @param lengthL: unused
  * @param lengthH: unused 
  * @return EP0BUF[0] == FIRMWARE_REVL, EP0BUF[1] == FIRMWARE_REVH
  */

#define VX_FW 			0x30              // Firmware revision


/** VX_PORTC writes value to port C
  * @param valueL: value to write
  * @return EP0BUF[0] = VX_PORTC
  */
#define VX_PORTC		    0x53




#endif
