// image.cpp : Defines the entry point for the console application
// bmp save routines.  image merge routines. jpg save routines
// uses jpeg-6b driver.
// Copyright 2006 Sensoray Corp Inc.
// Copyright 2007 Sensoray 
// DA: truncated for 2255 driver.  no file saving, jpeg etc..
#include "image.h"
#include <stdio.h>
#include <string.h>
//#include "driver.h"
unsigned short RGB2YCrCb16( int R, int G, int B )
{
    int Y  = ( 16842 * R + 33030 * G +  6422 * B + 1048576 ) >> ( 16 + 2 );
    int Cr = ( 28770 * R - 24117 * G -  4653 * B + 8388608 ) >> ( 16 + 3 );
    int Cb = ( -9699 * R - 19070 * G + 28770 * B + 8388608 ) >> ( 16 + 3 );

    Y  = Y  > ( 235 >> 2 ) ? ( 235 >> 2 ) : Y;
    Cr = Cr > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cr;
    Cb = Cb > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cb;

    Y  = Y  < ( 16 >> 2 ) ? ( 16 >> 2 ) : Y;    //lint !e701 shift left of signed 
    Cr = Cr < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cr;   //lint !e701 shift left of signed 
    Cb = Cb < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cb;   //lint !e701 shift left of signed 
    return ( Y << 10 ) + ( Cr << 5 ) + Cb;
}

/** converts RGB to YCrCb.  returns each YCrCb value separately
*  @param R red value
*  @param G green value
*  @param B blue value
*  @param pY pointer to converted Y value
*  @param pCr pointer to converted Cr value
*  @param pCb pointer to converted Cb value
*/
void RGB2YCrCb( int R, int G, int B, unsigned char *pY, unsigned char *pCr, unsigned char *pCb )
{
    int Y  = ( 16842 * R + 33030 * G +  6422 * B + 1048576 ) >> ( 16 ); //lint !e702 shift right of signed ...
    int Cr = ( 28770 * R - 24117 * G -  4653 * B + 8388608 ) >> ( 16 ); //lint !e702 shift right of signed ...
    int Cb = ( -9699 * R - 19070 * G + 28770 * B + 8388608 ) >> ( 16 ); //lint !e702 shift right of signed ...

    Y  = Y  > ( 235 ) ? ( 235 ) : Y;
    Cr = Cr > ( 240 ) ? ( 240  ) : Cr;
    Cb = Cb > ( 240 ) ? ( 240  ) : Cb;

    Y  = Y  < ( 16 ) ? ( 16 ) : Y;      //lint !e734 loss of precision 31 bits to 8
    Cr = Cr < ( 128 ) ? ( 128 ) : Cr;   //lint !e734 loss of precision 31 bits to 8
    Cb = Cb < ( 128 ) ? ( 128 ) : Cb;   //lint !e734 loss of precision 31 bits to 8

    *pY = (unsigned char )Y;
    *pCr = (unsigned char )Cr;
    *pCb = (unsigned char ) Cb;
    return;
}





/** reverse the RGB ordering in a line.  Used for the JPEG library.  pline is changed by this function
*  @param pline pointer to line
*  @param length length of line
*  @return none
*/
void reverse_rgb_line( unsigned char *pline, int length)
{
    int i;
    unsigned char tmp;

    for( i=0; i< length ;i+=3)
    {
        tmp = *(pline+i);
        *(pline+i) = *(pline+2+i);
        *(pline+2+i) = tmp;
    }
    return;
}

/** 65536 R = 76533(Y-16) + 104936 * (Cr-128)
*65536 G = 76533(Y-16) - 53451(Cr-128) - 25703(Cb -128)
* 65536 B = 76533(Y-16) + 132677(Cb-128)
* conversion YCrCb2RGB
* @param Y: y value
* @param Cr: Cr value
* @param Cb: Cb value
* @param *pR: returned R value
* @param *pG: returned G value
* @param *pB: returned B value
*/
void YCrCb2RGB( int Y, int Cr, int Cb, unsigned char *pR, unsigned char *pG, unsigned char *pB)
{
    int R,G,B;

    Y = Y - 16;
    Cr = Cr - 128;
    Cb = Cb - 128;

    R = (76533 * Y + 104936 * Cr) >> 16; 
    G = ((76533 * Y) - (53451 * Cr) - (25703 * Cb)) >> 16;   
    B = ((76533 * Y) + (132677 * Cb)) >> 16; 

    // Even with proper conversion, some values still need clipping.
    if (R > 255) R = 255;
    if (G > 255) G = 255;
    if (B > 255) B = 255;
    if (R < 0) R = 0;
    if (G < 0) G = 0;
    if (B < 0) B = 0;

    *pR = R;  //lint !e734 loss of precision
    *pG = G;  //lint !e734 loss of precision
    *pB = B;  //lint !e734 loss of precision
    return;
}


/** YCrCb2RGB_fp floating point YCrCb conversion
* @param y y value
* @param Cr Cr value
* @param Cb Cb value
* @param pR return Red value
* @param pG pointer to G value
* @param pB pointer to B value
*/
void YCrCb2RGB_fp(int y, int Cr, int Cb, unsigned char *pR, unsigned char *pG, unsigned char *pB)
{
    double r, g, b;
    /* Commented out by Phillip 5/28/05
    r = y + (1.370705 * (v-128));
    g = y - (0.698001 * (v-128)) - (0.337633 * (u-128));
    b = y + (1.732446 * (u-128));

    r = (220*r)/256;
    g = (220*g)/256;
    b = (220*b)/256;
    */

    //////////////////////////////////
    // Added - Phillip 5/28/05
    y = y - 16;
    Cr = Cr - 128;
    Cb = Cb - 128;

    r = 1.164 * y + 1.596 * Cr;
    g = 1.164 * y - 0.813 * Cr - 0.391 * Cb;
    b = 1.164 * y + 2.018 * Cb;

    if (r > 255) r = 255;
    if (g > 255) g = 255;
    if (b > 255) b = 255;

    if (r < 0) r = 0;
    if (g < 0) g = 0;
    if (b < 0) b = 0;

    *pR = (unsigned char) r;
    *pG = (unsigned char) g;
    *pB = (unsigned char) b;
    return;
}





/** converts yuv planar image to rgb image in memory
*  @param in pointer to input image
*  @param out pointer to output image
*  @param height height of image
*  @param width width of image
*  @return 0 on success
*/
int planar422p_to_rgb_mem(  unsigned char *in, unsigned char *out, int width, int height)
{
    unsigned long i;
    unsigned char *pY;
    unsigned char *pCb;
    unsigned char *pCr;
    unsigned long  size = height * width;
    unsigned char r,b,g;
    pY = (unsigned char *) in;
    pCb =(unsigned char *) in + height * width;
    pCr = (unsigned char *) in + height * width +  (height * width /2);
    //printf("size %d\n", size);
    for( i=0;i<size*2;i+=3) {
        YCrCb2RGB( *pY, *pCr, *pCb, &r,&g,&b);
        out[i] = r;
        out[i+1] = g;
        out[i+2] = b;
        pY++;
        pCr++;
        pCb++;
    }
    return 0; 
}


void planar422p_to_yuy2(  unsigned char *in, unsigned char *out, int width, int height)
{
    unsigned char *pY;
    unsigned char *pCb;
    unsigned char *pCr;
    unsigned long  size = height * width;
    unsigned int i;
    pY = (unsigned char *) in;
    pCr =(unsigned char *) in + height * width;
    pCb = (unsigned char *) in + height * width +  (height * width /2);
    //printf("pY %x pCr %x pCr-1 %x pCb[-1] %x, %x\n", *pY, *pCr, *(pCr-1), *(pCb-1), *pCb);
    for( i=0;i<size*2;i+=4) {
        out[i] = *pY++;
        out[i+1] = *pCr++;
        out[i+2] = *pY++;
        out[i+3] = *pCb++;
    }
    return;
}

void planar422p_to_yuv(  unsigned char *in, unsigned char *out, int width, int height)
{
    unsigned char *pY;
    unsigned char *pYEND;
    unsigned char *pCb;
    unsigned char *pCr;
    unsigned long  size = height * width;
    unsigned long i;
    pY = (unsigned char *) in;
    pCr =(unsigned char *) in + (height * width);
    pCb = (unsigned char *) in + (height * width) +  (height * width /2);
    pYEND = pCr;
    for( i=0; pY<pYEND; i+=6) {
        out[i] = *pY++;
        out[i+1] = *pCr;
        out[i+2] = *pCb;
        out[i+3] = *pY++;        
        out[i+4] = *pCr++;
        out[i+5] = *pCb++;        

    }

    return;
}




/** saves an image to bmp file
*  @param filename is the full file path to save( minus the extension)
*  @param *image, pointer to image data in Y Cr Cb format
*  @param type is the type( 0-- BMP,  1--ppm)
*  @return 0 on success, -1 if file invalid
*/
int save_image_uncompressed(const unsigned char *image, const char *szFilename, image_info_t *info, int type)
{
    char ppmheader[256];

    FILE *fptr;
    int val;
    int rlen; // row length or number of columns( unsigned chars== pixels * unsigned chars per pixel)
    int clen; // column length or number of rows
    int bpp; // unsigned chars per pixel! not bits
    int i;

    rlen = info->stride;

    clen = info->height;
    bpp = info->stride / info->width;

    fptr = fopen( szFilename, "wb");

    if (fptr == NULL) {
        printf("could not open\n");
        return -1;
    }

    if (type == 1)
    {
        sprintf( ppmheader, "P6\n#ppm image\n%d %d\n255\n", info->width, info->height);
        fwrite( ppmheader, 1, strlen(ppmheader), fptr);
        // write out the rows
        for ( i=0; i< clen; i++)
        {
            fwrite( &image[rlen*i], 1, rlen, fptr);
        }
    }
    else
    {
        val = 0x4d42;                   fwrite(&val,1,2,fptr); //bmp signature
        val = (rlen * clen / bpp) + 54; fwrite(&val,1,4,fptr); //size of bmp file
        val = 0;                        fwrite(&val,1,2,fptr); //must be 0
        val = 0;                        fwrite(&val,1,2,fptr); //must be 0
        val = 54;                       fwrite(&val,1,4,fptr); //offset to image start
        val = 40;                       fwrite(&val,1,4,fptr); //must be 40
        val = rlen / bpp;               fwrite(&val,1,4,fptr); //image width in pixels
        val = clen ;                    fwrite(&val,1,4,fptr); //image height in pixels
        val = 1;                        fwrite(&val,1,2,fptr); //must be 1
        val = bpp * 8;                  fwrite(&val,1,2,fptr); //bits per pixel
        val = 0;                        fwrite(&val,1,4,fptr); //compression (none)
        val = (rlen * clen / bpp);      fwrite(&val,1,4,fptr); //size of image data
        val = 2835;                     fwrite(&val,1,4,fptr); //horizontal pixels/m
        val = 2835;                     fwrite(&val,1,4,fptr); //vertical pixels/m
        val = 0;                        fwrite(&val,1,4,fptr); //colors in image, or 0
        val = 0;                        fwrite(&val,1,4,fptr); //important colors,or 0

        for (i= clen-1; i>= 0; i--)  //rows
        {
            fwrite( &image[i*rlen], 1, rlen, fptr);
        }
        /* for regular order
        for (i= 0; i<clen; i++)  //rows
        {
        fwrite( &image[i*rlen], 1, rlen, fptr);
        }
        */
    }

    fclose( fptr);
    return 0;
}



/** converts yuv image to rgb image in memory
*  @param in pointer to input image
*  @param out pointer to output image
*  @param height height of image
*  @param width width of image
*  @return 0 on success
*/
int yuv2rgb_mem( const unsigned char *in, unsigned char *out, int height, int width)
{
    int i;
    unsigned char y,Cr,Cb;
    unsigned char r,g,b;
    for ( i = 0; i< height*width * 3; i+=3) {
        y = in[i]; 
        Cr = in[i+1];
        Cb = in[i+2];
        //#define USE_FP_CONVERSION
#ifndef USE_FP_CONVERSION
        YCrCb2RGB( y,Cr,Cb,&r,&g,&b);
#else
        YCrCb2RGB_fp(y, Cr, Cb, &r, &g, &b);
#endif
        out[i]  = r;
        out[i+1] = g;
        out[i+2] = b;

    }

    return 0; 

}


/** converts yuv image to rgb image in memory
*  @param in pointer to input image
*  @param out pointer to output image
*  @param height height of image
*  @param width width of image
*  @return 0 on success
*/
int yuv2rgb565_mem( const unsigned char *in, unsigned char *out, int height, int width)
{
    int i;
    int k=0;
    unsigned char y,Cr,Cb;
    unsigned char r,g,b;
    unsigned short rgbbytes;

    for ( i = 0; i< height*width * 3; i+=3) {
        y = in[i]; 
        Cb = in[i+1];
        Cr = in[i+2];
        //#define USE_FP_CONVERSION
#ifndef USE_FP_CONVERSION
        YCrCb2RGB( y,Cr,Cb,&r,&g,&b);
#else
        YCrCb2RGB_fp(y, Cr, Cb, &r, &g, &b);
#endif
        r = r >> 3;
        g = g >> 2;
        b = b >> 3;
        rgbbytes = b + (g<<5) + (r<<(5+6));
        out[k++] = rgbbytes & 0xff;
        out[k++] = (rgbbytes >> 8) & 0xff;
    }

    return 0; 
}


void planar422p_to_rgb565(  unsigned char const *in, unsigned char *out, int width, int height)
{
    unsigned char *pY;
    unsigned char *pYEND;
    unsigned char *pCb;
    unsigned char *pCr;
    unsigned char Cr,Cb,Y,r,g,b;
    unsigned long k = 0;
    unsigned short rgbbytes;
    pY = (unsigned char *) in;
    pCb =(unsigned char *) in + (height * width);
    pCr = (unsigned char *) in + (height * width) +  (height * width /2);
    pYEND = pCb;
    while( pY<pYEND ) {
        Y = *pY++;
        Cr = *pCr;
        Cb = *pCb;
#ifndef USE_FP_CONVERSION
        YCrCb2RGB( Y,Cr,Cb,&r,&g,&b);
#else
        YCrCb2RGB_fp(Y, Cr, Cb, &r, &g, &b);
#endif
        r = r >> 3;
        g = g >> 2;
        b = b >> 3;
        rgbbytes = b + (g<<5) + (r<<(5+6));        
        out[k++] = rgbbytes & 0xff;
        out[k++] = (rgbbytes >> 8) & 0xff;

        Y = *pY++;
        Cr = *pCr++;
        Cb = *pCb++;

#ifndef USE_FP_CONVERSION
        YCrCb2RGB( Y,Cr,Cb,&r,&g,&b);
#else
        YCrCb2RGB_fp(Y, Cr, Cb, &r, &g, &b);
#endif
        r = r >> 3;
        g = g >> 2;
        b = b >> 3;
        rgbbytes = b + (g<<5) + (r<<(5+6));
        out[k++] = rgbbytes & 0xff;
        out[k++] = (rgbbytes >> 8) & 0xff;        
    }
    return;
}


void planar422p_to_rgb24(  const unsigned char *in, unsigned char *out, int width, int height)
{
    unsigned char *pY;
    unsigned char *pYEND;
    unsigned char *pCb;
    unsigned char *pCr;
    unsigned char Cr,Cb,Y,r,g,b;
    unsigned long k = 0;
    pY = (unsigned char *) in;
    pCb =(unsigned char *) in + (height * width);
    pCr = (unsigned char *) in + (height * width) +  (height * width /2);
    pYEND = pCb;
    while( pY<pYEND ) {
        Y = *pY++;
        Cr = *pCr;
        Cb = *pCb;
#ifndef USE_FP_CONVERSION
        YCrCb2RGB( Y,Cr,Cb,&r,&g,&b);
#else
        YCrCb2RGB_fp(Y, Cr, Cb, &r, &g, &b);
#endif
        out[k++] = b;
        out[k++] = g;
        out[k++] = r;
        if (pY >= pYEND) {
            break;
        }
        Y = *pY++;
        Cr = *pCr++;
        Cb = *pCb++;
        

#ifndef USE_FP_CONVERSION
        YCrCb2RGB( Y,Cr,Cb,&r,&g,&b);
#else
        YCrCb2RGB_fp(Y, Cr, Cb, &r, &g, &b);
#endif
        out[k++] = b;
        out[k++] = g;
        out[k++] = r;        
    }

    return;
}

void planar422p_to_rgb32(  const unsigned char *in, unsigned char *out, int width, int height)
{
    unsigned char *pY;
    unsigned char *pYEND;
    unsigned char *pCb;
    unsigned char *pCr;
    unsigned char Cr,Cb,Y,r,g,b;
    unsigned long k = 0;
    pY = (unsigned char *) in;
    pCb =(unsigned char *) in + (height * width);
    pCr = (unsigned char *) in + (height * width) +  (height * width /2);
    pYEND = pCb;
    while( pY<pYEND ) {
        Y = *pY++;
        Cr = *pCr;
        Cb = *pCb;
#ifndef USE_FP_CONVERSION
        YCrCb2RGB( Y,Cr,Cb,&r,&g,&b);
#else
        YCrCb2RGB_fp(Y, Cr, Cb, &r, &g, &b);
#endif
        out[k++] = b;
        out[k++] = g;
        out[k++] = r;
        out[k++] = 0;
        if (pY >= pYEND) {
            break;
        }
        Y = *pY++;
        Cr = *pCr++;
        Cb = *pCb++;
        

#ifndef USE_FP_CONVERSION
        YCrCb2RGB( Y,Cr,Cb,&r,&g,&b);
#else
        YCrCb2RGB_fp(Y, Cr, Cb, &r, &g, &b);
#endif
        out[k++] = b;
        out[k++] = g;
        out[k++] = r;        
        out[k++] = 0;
    }

    return;
}


//DA: only use this if MMX not working
void grayscale_rgb565(  const unsigned char *in, unsigned char *out, int width, int height)
{
    int i=0;
    int gray;
    int rgbbytes;
    int k=0;
    int r,g,b;
    while( i< width*height)
    {
        gray = in[i++];
        r = gray >> 3;
        b = gray >> 3;
        g = gray >> 2;
        rgbbytes = b + (g<<5) + (r<<(5+6));
        
        out[k++] = rgbbytes & 0xff;
        out[k++] = (rgbbytes >> 8) & 0xff;
    }
}


void grayscale_rgb24(  const unsigned char *in, unsigned char *out, int width, int height)
{
    int i=0;
    int gray;
    int rgbbytes;
    int k=0;
    while( i< width*height)
    {
        gray = in[i++];
        out[k++] = gray;
        out[k++] = gray;
        out[k++] = gray;
    }
}

void grayscale_rgb32(  const unsigned char *in, unsigned char *out, int width, int height)
{
    int i=0;
    int gray;
    int rgbbytes;
    int k=0;
    while( i< width*height)
    {
        gray = in[i++];
        out[k++] = gray;
        out[k++] = gray;
        out[k++] = gray;
        out[k++] = 0;
    }
}
