/**********************************************************
 * This file contains special types, constants
 * for use with Sensoray sx2255 driver
 *
 * Copyright (C) Sensoray 2007
 **********************************************************/

#ifndef SX2255_H
#define SX2255_H

#define MAX_CHANNELS 4

typedef	float				REAL32;
typedef	double				REAL64;
typedef	unsigned char		UINT8;
typedef	char				SINT8;
typedef	unsigned short		UINT16;
typedef	short				SINT16;
typedef void				*PVOID;
typedef unsigned long		ULONG;
typedef unsigned char		UCHAR;
typedef	int		            INT32;
typedef void *              HANDLE;


#if ( defined(WIN32) || defined(OSTYPE_WINDOWS) )
typedef	INT32				SINT32;
typedef	unsigned int		UINT32;
typedef HANDLE              HDEVICE;
#elif ( defined(_LINUX) || defined( OSTYPE_LINUX) )
// Linux, no bitmapinfo

#define STATUS_NO_MEMORY -1003
typedef long                    LONG;
typedef	unsigned int		UINT32;
typedef	int					SINT32;
typedef void				VOID;
typedef void *				LPVOID;
typedef int					BOOLEAN;
typedef int					BOOL;
typedef unsigned int		DWORD;
typedef unsigned int		NTSTATUS;
typedef int                 HDEVICE;
typedef char                BYTE;
typedef int                 WORD;
#endif


typedef struct tagRGBQUAD {
        char    rgbBlue;
        char    rgbGreen;
        char    rgbRed;
        char    rgbReserved;
} RGBQUAD;


typedef struct tagBITMAPINFOHEADER{
        ULONG      biSize;
        long       biWidth;
        long       biHeight;
        UINT16     biPlanes;
        UINT16     biBitCount;
        ULONG      biCompression;
        ULONG      biSizeImage;
        long       biXPelsPerMeter;
        long       biYPelsPerMeter;
        ULONG      biClrUsed;
        ULONG      biClrImportant;
} BITMAPINFOHEADER, *LPBITMAPINFOHEADER, *PBITMAPINFOHEADER;

typedef struct tagBITMAPINFO {
    BITMAPINFOHEADER    bmiHeader;
    RGBQUAD             bmiColors[1];
} BITMAPINFO, *LPBITMAPINFO, *PBITMAPINFO;



#define SYS_GRABBERS            32       //max frame grabbers in the system;

#if defined (OSTYPE_LINUX) 
#define SYS_FRAMES          6		//max frames allowed;
#elif defined (OSTYPE_WINDOWS)
#if defined (DRIVER_DSHOW)
#define SYS_FRAMES          6
#else
#define SYS_FRAMES           64		//max frames alloed;
#endif
#endif
   


typedef struct {                        // frame structure;
    void *pdata;						// pointer to image data;
} FRAME;
   

typedef struct {               //image buffer structure;
    ULONG dwFrames;            //number of frames in buffer;
    FRAME frame[SYS_FRAMES];   //array of FRAME structures;
    PVOID lpbmi[SYS_FRAMES];   // pointer to BITMAPINFO structure;
	int size;
} BUFFER;

typedef struct {
 UINT32   format;   //input video format (NTSC, PAL, SECAM)
 UINT32   scale;   //output video scale
 UINT32   color;   //output video color format
 UINT32   fdec;   //frame decimation
 UINT32   bright;   //brightness
 UINT32   contrast;  //contrast
 UINT32   saturation;  //saturation
 UINT32   hue;   //hue (NTSC only)
 UINT32   single;   //capture 1 frame at a time (!=0), continuously (==0)
} MODE2255;

typedef struct {
 UINT32 serial_number;
 UINT8	month;
 UINT8  day;
 UINT8  year;
} SN2255;

 
//predefined settings
#define FORMAT_NTSC    1
#define FORMAT_PAL     2
 
#define SCALE_4CIFS    1 //640x480(NTSC) or 704x576(PAL)
#define SCALE_2CIFS    2 //640x240(NTSC) or 704x288(PAL)
#define SCALE_1CIFS    3 //320x240(NTSC) or 352x288(PAL)
#define SCALE_4CIFSI   4 //640x480(NTSC) or 704x576(PAL) interpolated
#define SCALE_1CIFSF   5 // 1CIF 60Hz (all fields)
 
#define COLOR_YUVPL    1 //YUV planar
#define COLOR_YUVPK    2 //YUV packed
#define COLOR_RGB      3 //RGB (packed, GBR, Windows .bmp compatible)
#define COLOR_Y8       4 //monochrome
#define COLOR_JPG      5 //JPEG compressed
#define MASK_COLOR	    0xff // color sitting mask
#define MASK_JPG_QUALITY 0xff00 // jpeg quality mask
#define MASK_INPUT_TYPE  0xf0000

#define FDEC_1     1     //capture every frame
#define FDEC_2     2     //capture every 2nd frame
#define FDEC_3     3     //capture every 3rd frame
#define FDEC_5     5     //capture every 5th frame

/*-------------------------------------------------------
 * Default mode parameters.
 *-------------------------------------------------------*/
#define DEF_SCALE               SCALE_4CIFS       //default scaling mode, full image;
#define DEF_COLOR               COLOR_RGB         //default color mode, RGB output;
#define DEF_FDEC                FDEC_1
#define DEF_BRIGHT              0
#define DEF_CONTRAST            0x5c
#define DEF_SATURATION          0x80
#define DEF_HUE                 0

#define DEF_MODE_NTSC_CONT  FORMAT_NTSC, DEF_SCALE, DEF_COLOR, DEF_FDEC, DEF_BRIGHT, DEF_CONTRAST, DEF_SATURATION, DEF_HUE, 0
#define DEF_MODE_PAL_CONT   FORMAT_PAL, DEF_SCALE,  DEF_COLOR, DEF_FDEC, DEF_BRIGHT, DEF_CONTRAST, DEF_SATURATION, DEF_HUE, 0
#define DEF_MODE_NTSC_SING  FORMAT_NTSC, DEF_SCALE, DEF_COLOR, DEF_FDEC, DEF_BRIGHT, DEF_CONTRAST, DEF_SATURATION, DEF_HUE, 1
#define DEF_MODE_PAL_SING   FORMAT_PAL, DEF_SCALE,  DEF_COLOR, DEF_FDEC, DEF_BRIGHT, DEF_CONTRAST, DEF_SATURATION, DEF_HUE, 1

/*-------------------------------------------------------
 * Error codes
 *-------------------------------------------------------*/
#define SUCCESS                 0
#define ERR_UNKNOWN             -1

#endif
