/**********************************************************
 * This file contains special types, constants
 * for use with Sensoray S2255
 *
 * Copyright (C) Sensoray 2007
 **********************************************************/
// INTERNAL USE ONLY.  exposed to Sensoray, not the customer

#ifndef SX2255I_H
#define SX2255I_H
#include "s2255mod.h"

#define IN_DATA_TOKEN 0x2255c0de     //command token
#define CMD_2255  0xc2255000     //command base
#define CMD_SET_MODE (CMD_2255 | 0x10)   //set mode
#define CMD_START  (CMD_2255 | 0x20)   //start capture
#define CMD_STOP  (CMD_2255 | 0x30)   //stop capture
// future use, status command
#define CMD_STATUS  (CMD_2255 | 0x40)   //return status

typedef struct {
 UINT32   format;   //input video format (NTSC, PAL, SECAM)
 UINT32   scale;   //output video scale
 UINT32   color;   //output video color format
 UINT32   fdec;   //frame decimation
 UINT32   bright;   //brightness
 UINT32   contrast;  //contrast
 UINT32   saturation;  //saturation
 UINT32   hue;   //hue (NTSC only)
 UINT32   single;   //capture 1 frame at a time (!=0), continuously (==0)
 UINT32   usb_block; // block size
 UINT32   restart; // if DSP requires restart( internal use only)
} MODE2255I;
    
typedef struct {                        // frame structure;
    ULONG size;
	ULONG read_state;
    UINT32 ts;
    void *lpvbits;						// pointer to image data;
    void *sysbuf;						// pointer to sysbuffer
	ULONG cur_size;
    PVOID pMDL;                         // internal use
	PVOID pMDL_bmi;                      // internal use
	char  *lock_buf;  // locked data
	LPBITMAPINFO lock_lpbmi;					// pointer to locked lpmbi for this frame
} FRAMEI;
   
typedef struct {                        //image buffer structure;
    ULONG dwFrames;                     //number of frames in buffer;
    FRAMEI frame[SYS_FRAMES];            //array of FRAME structures;
	LPBITMAPINFO lpbmi[SYS_FRAMES];
} BUFFERI;


#define DEF_MODEI_NTSC_CONT  FORMAT_NTSC, DEF_SCALE, DEF_COLOR, DEF_FDEC, DEF_BRIGHT, DEF_CONTRAST, DEF_SATURATION, DEF_HUE, 0, 4096,0
#define DEF_MODEI_PAL_CONT   FORMAT_PAL, DEF_SCALE,  DEF_COLOR, DEF_FDEC, DEF_BRIGHT, DEF_CONTRAST, DEF_SATURATION, DEF_HUE, 0, 4096,0
#define DEF_MODEI_NTSC_SING  FORMAT_NTSC, DEF_SCALE, DEF_COLOR, DEF_FDEC, DEF_BRIGHT, DEF_CONTRAST, DEF_SATURATION, DEF_HUE, 1, 4096,0
#define DEF_MODEI_PAL_SING   FORMAT_PAL, DEF_SCALE,  DEF_COLOR, DEF_FDEC, DEF_BRIGHT, DEF_CONTRAST, DEF_SATURATION, DEF_HUE, 1, 4096,0

/*lint +e761*/

#endif
