// Sensoray 2255 driver core module.
// Copyright 2007-2012 Sensoray Company Inc.
// D.A.

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
#include <linux/config.h>
#endif
#include <linux/module.h>
#include <linux/mman.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kref.h>
#include <linux/vmalloc.h>
#include <asm/uaccess.h>
#include <linux/usb.h>
#include <linux/dma-mapping.h>
#include <linux/highmem.h>
#include <linux/kthread.h>

#include "s2255core.h"
#include "s2255mod.h"
#include "2255vx.h"
#include "debug.h"
#include "s2255_i.h"
#include "size.h"
#define PREFIX_SIZE         512             //frame prefix size (sent once every frame)
// Because the channels were printed in reverse order than originally planned
static ULONG G_chnmap[MAX_CHANNELS]= {3,2,1,0};

#define S2255_MARKER_FRAME	0x2255DA4AL
#define S2255_MARKER_RESPONSE	0x2255ACACL

#ifdef WIN32
#include "usb_base.h"
#pragma LOCKEDCODE
#endif

// create the core structures and init the board
#define S2255_SETMODE_TIMEOUT   500



void S2255CORE_read_video_callback(HANDLE board, S2255_USB_PIPE_INFO* pipe_info);

#define IC2_SUPPORT
// linux must have kmalloc'd continuous memory for ALL USB transfers
#define FREE_USB_MEM2(val) s2255_free_dma_mem(val)
#define ALLOC_USB_MEM2(val) s2255_alloc_dma_mem(val)



PVOID S2255CORE_board_init( int product_id, PVOID pdx)
{
	PVPAR v;
	int index;
	int j;
	MODE2255I mode_def = { DEF_MODEI_NTSC_CONT };

	if (product_id != 2255) {
		errprint0("bad product ID!\n");
		return NULL;
	}

	v = (PVPAR) s2255_alloc_mem( sizeof( VPAR));
	if (v == NULL) {
		errprint0("OUT of memory!\n");
		return NULL;
	}

	s2255_memset(v, sizeof( VPAR), 0);
    v->cmdbuf = (unsigned char *)ALLOC_USB_MEM2(512);
    if (v->cmdbuf == NULL) {
        printk("out of memory!\n");
        return NULL;
    }
	v->pdx = pdx;
	spin_lock_init(&v->slock);
	for( j=0;j<MAX_CHANNELS;j++) {
		v->b_acquire[j] = 0;
		v->mode[j] = mode_def;
		v->req_image_size[j] = get_transfer_size( &mode_def);
		v->buffers_reg[j] = 0;
	}
	//dbgprint0("S2255: init 3\n");
	for( j=0;j<MAX_PIPE_BUFFERS;j++) {
		for ( index = 0 ; index < MAX_PIPE_NUMBER ; index ++ ) {
			S2255_USB_PIPE_INFO* pipe_info = &v->UsbPipes[index][j];
			s2255_memset(pipe_info, sizeof(S2255_USB_PIPE_INFO), 0);
            init_waitqueue_head(&pipe_info->hStopEvent);
			pipe_info->dwState = 0;
			pipe_info->dwPrevState = 0;
			pipe_info->pdx = pdx;                               /* pointer to the device context */
			pipe_info->hbd = v; 
			pipe_info->cur_transfer_size = DEF_USB_BLOCK;
			pipe_info->max_transfer_size = DEF_USB_BLOCK;
			if (pipe_info->cur_transfer_size > pipe_info->max_transfer_size)
				pipe_info->cur_transfer_size = pipe_info->max_transfer_size;

			pipe_info->pTransferBuffer = (UINT8 *) ALLOC_USB_MEM2( pipe_info->max_transfer_size ); //256 k max;
			if (pipe_info->pTransferBuffer == NULL) {
				errprint0("out of memory!\n");
				return NULL;
			}
			if (index == INDEX_READ_PIPE) {
				pipe_info->pCallBack = S2255CORE_read_video_callback;
			}
		}
	}
	// set the address
	v->dbgaddr = &v->UsbPipes[0][0];

	for(j = 0; j < MAX_CHANNELS; j++) {
		S2255CORE_create_sys_buffers(v, j);
	}

	dbgprint0("S2255: board initialized\n");
	return v;
}



int S2255CORE_board_open( HANDLE board)
{
	PVPAR v = (PVPAR) board;
    unsigned long flags;
	int j;
	MODE2255I mode_def = { DEF_MODEI_NTSC_CONT };

	if (v== NULL) {
		errprint1( "S2255: board open: failed! %p", board);     
		return -1;
	}
#if DBG
	dbgprint1( "S2255: board open: %p", board);
#endif

#if defined (OS_LINUX)    
	for( j=0;j<MAX_CHANNELS;j++) {
		v->b_acquire[j] = 0;
		v->mode[j] = mode_def;
		v->req_image_size[j] = get_transfer_size( &mode_def);
		v->buffers_reg[j] = 0;
	}
#endif

    spin_lock_irqsave(&v->slock, flags);

	if (v->is_open == 1) {
		dbgprint0("s2255: board already open\n");
        spin_unlock_irqrestore(&v->slock, flags);
		return -1;
	}
	v->is_open = 1;
	v->map_failed = 0;
	v->b_searching = 1;
	v->frame_count = 0;
    spin_unlock_irqrestore(&v->slock, flags);





	for( j=0;j<MAX_CHANNELS;j++) {
		int chn;
		chn = G_chnmap[j];
		v->mode[j].restart = 1;
        (void) S2255CORE_set_mode( v, chn, &v->mode[j]);
	}
	return 0;
}


int S2255CORE_board_close( HANDLE board)
{
	UINT32 i;
    unsigned long flags;
	PVPAR v = (PVPAR) board;
#if DBG
	dbgprint1("S2255: board close: %p", board);
#endif

	for( i=0;i<MAX_CHANNELS;i++) {
		S2255CORE_stop_acquire( board, i);  // 
	}

    spin_lock_irqsave(&v->slock, flags);
	if (!v->is_open ) {
        spin_unlock_irqrestore(&v->slock, flags);
		dbgprint0("not open\n");
		return -1;
	}
	v->is_open = 0;
    spin_unlock_irqrestore(&v->slock, flags);

#if DBG
	dbgprint0("stopped\n");
#endif
	return 0;
}

int S2255CORE_board_shutdown( HANDLE board)
{
	UINT32 i;
	int j;
	PVPAR v = (PVPAR) board;
    unsigned long flags;
#if DBG
	dbgprint1("board shutdown BOARD: %p\n", board);
#endif
	if (v == NULL)
		return -1;

	dbgprint1("board shutdown pdx %p\n", v->pdx);

	if (v->pdx == NULL) {
		return -1;
	}

	for( i=0;i<MAX_CHANNELS;i++) {
		S2255CORE_stop_acquire( board, i);
	}

	s2255_stop_readpipe(v->pdx);
	s2255_vendor_request(v->pdx, 0x40, 0x0b0b, 0x0b01, NULL, 0, 1);
	s2255_sleep(10);
	s2255_vendor_request(v->pdx, 0x50, 0x0000, 0x0000, NULL, 0, 1);
	s2255_sleep(600);

	for( j=0;j<MAX_PIPE_BUFFERS;j++) {
		for ( i = 0 ; i < MAX_PIPE_NUMBER ; i ++ ) 
		{
			S2255_USB_PIPE_INFO* pipe_info = &v->UsbPipes[i][j];
			if (pipe_info->pTransferBuffer) {
				FREE_USB_MEM2( pipe_info->pTransferBuffer);
				pipe_info->pTransferBuffer = NULL;
			}
		}
	}
	//KdPrint(("spin lock done\n"));
	// remove all the locked user buffer, for all channels
	for( j=0;j< MAX_CHANNELS;j++) {
		for( i=0; i<SYS_FRAMES;i++) {
			if (v->buffer[j].frame[i].pMDL) {
				s2255_unlock_buffer( v->buffer[j].frame[i].pMDL);
				v->buffer[j].frame[i].lock_buf = NULL;
				v->buffer[j].frame[i].pMDL = NULL;
			}
		}
	}

	spin_lock_irqsave(&v->slock, flags);
	v->is_open = 0;
	v->is_shutdown = 1;
    spin_unlock_irqrestore(&v->slock, flags);
	for(j = 0; j < MAX_CHANNELS; j++) {
		S2255CORE_release_sys_buffers(v, j);
	}

    FREE_USB_MEM2(v->cmdbuf);
	s2255_free_mem( v);
	return 0;
}


/** read single byte from I2C
 *  @param pdx pointer to device extension
 *  @param addr address of I2C device to read
 *  @param subaddr subaddress to read from
 *  @param data data to write to
 *  @param datalen length of data
 *  @return value read from I2C (or 0xcd if invalid)
 */
int S2255CORE_read_i2c_burst( HANDLE board, UINT8 addr, UINT16 subaddr, UINT8 *data, UINT8 datalen)
{
	int res;
	PVPAR v = (PVPAR )board;
	if (v==NULL)
		return -1;

	res = s2255_I2C_ReadRegisterBurst( v->pdx, addr,subaddr, data, datalen );
	dbgprint2("data[0], data[1]: %x, %x\n", data[0], data[1]);	
	return res;
}

#ifdef I2C_SUPPORT

/** write burst of data to i2c device
 *  @param pdx pointer to device extension
 *  @param addr address of I2C device to write to
 *  @param length of data to write
 *  @param data pointer to data to write( should include subaddress if necessary)
 *  @return I2C_WriteReg2 result
 */

int S2255CORE_write_i2c_burst( HANDLE board, UINT8 addr, UINT8 length, UINT8 *data)
{
	int res;
	PVPAR v = (PVPAR) board;
	if (v == NULL)
		return -1;
	dbgprint4("i2c write burst ADDR %x LEN %x, DATA0 %x DATA1 %x\n",addr,length,data[0],data[1]);
	res = s2255_I2C_WriteReg2( v->pdx, addr, length, data);
	return res;
}	

int S2255CORE_write_i2c_burst_eeprom( HANDLE board, UINT8 addr, UINT8 length, UINT8 *data)
{
	int res;
	PVPAR v = (PVPAR) board;
	if (v == NULL)
		return -1;
	dbgprint4("i2c write burst ADDR %x LEN %x, DATA0 %x DATA1 %x\n",addr,length,data[0],data[1]);
	res = s2255_I2C_WriteReg3( v->pdx,(UINT8) addr, length, data);
	return res;
}	



/** write single byte to i2c device
 *  @param pdx pointer to device extension
 *  @param addr address of I2C device to write to
 *  @param subaddr subaddress to write to
 *  @param value value to write
 *  @return I2C_WriteRegister result
 */
int S2255CORE_write_i2c( HANDLE board, UINT8 addr, UINT16 subaddr, UINT8 value)
{
	int res;
	PVPAR v = (PVPAR) board;
	if (v == NULL)
		return -1;
	dbgprint3("i2c write %x %x %x\n",addr,subaddr,value);
	res = s2255_I2C_WriteRegister( v->pdx, addr, subaddr, value);
	return res;
}

/** read single byte from I2C
 *  @param pdx pointer to device extension
 *  @param addr address of I2C device to read
 *  @param subaddr subaddress to read from
 *  @return value read from I2C (or 0xcd if invalid)
 */
int S2255CORE_read_i2c( HANDLE board, UINT8 addr, UINT16 subaddr)
{
	PVPAR v = (PVPAR )board;
	UINT8 reg = 0;
	if (v == NULL)
		return -1;
	s2255_I2C_ReadRegister( v->pdx, addr,subaddr, &reg );
	dbgprint1("i2c read %x\n", reg);
 	return reg;
}

/** read single byte from I2C
 *  @param pdx pointer to device extension
 *  @param addr address of I2C device to read
 *  @param subaddr subaddress to read from
 *  @param data data to write to
 *  @param datalen length of data
 *  @return value read from I2C (or 0xcd if invalid)
 */
int S2255CORE_read_i2c_burst( HANDLE board, UINT8 addr, UINT16 subaddr, UINT8 *data, UINT8 datalen)
{
	int res;
	PVPAR v = (PVPAR )board;
	if (v == NULL)
		return -1;
  	dbgprint2("i2c read register burst device %x, reg %x\n",addr,subaddr);
	res = s2255_I2C_ReadRegisterBurst( v->pdx, addr,subaddr, data, datalen );
	dbgprint2("data[0], data[1]: %x, %x\n", data[0], data[1]);
 
	return res;
}

/** read I2c without specify a subaddress, aka burst read.
 *  @param pdx pointer to device extension
 *  @param addr I2c main address
 *  @param datalen data length to read
 *  @param data pointer to data
 *  @return I2C_ReadReg2 result
 */

int S2255CORE_read_i2c_nostart( HANDLE board, UINT8 addr, UINT8 datalen, UINT8 *data)
{
	int res;
	PVPAR v = (PVPAR) board;
 	if (v == NULL)
		return -1;
	res = s2255_I2C_ReadReg2(v->pdx, addr, datalen, data );
	if(datalen > 0) {
		dbgprint3("i2c read no start--addr:%x--len %x d[0]%x\n", addr, datalen, data[0]);
	}
	return res;
}

#endif

/** retrieve FX2 firmware version
 * @param pdx pointer to device extension
 * @return -1 for fail, else returns firmware version as an int(16 bits)
 */
int S2255CORE_get_fx2fw( HANDLE board)
{
	int fw;
	int ret;
	unsigned char transBuffer[64];
	PVPAR v = (PVPAR) board;
	if (v == NULL)
		return -1;
	ret = s2255_vendor_request(v->pdx, VX_FW, 0, 0, transBuffer, 2, DIR_IN);
	if (ret < 0) {
		dbgprint1("get fw error: %x\n",ret);
	}
	fw = transBuffer[0] + (transBuffer[1] << 8) ;
	dbgprint2("Get FW %x %x\n", transBuffer[0], transBuffer[1]);
	return fw;
}

int S2255CORE_set_buffer_dbg( HANDLE board, PVOID pdin, ULONG cbin)
{
	PVPAR v = (PVPAR ) board;
	if (v == NULL)
		return -1;
	dbgprint2( "setting buffer %p, size %lu\n", pdin, cbin);
	v->pBuffer = pdin;
	v->buf_size = cbin;
	return 0;
}



int S2255CORE_dqbuf( HANDLE board, ULONG chn, ULONG idx )
{
	PVPAR v = (PVPAR ) board;
	ULONG channel;
	if (v == NULL)
		return -1;
	if (chn >= MAX_CHANNELS) {
		return -1;
	}
	channel = G_chnmap[chn];
	if (idx >= SYS_FRAMES) {
		return -1;
	}
	// always reset the buffers in case DSP didn't transfer all data
	v->buffer[channel].frame[idx].read_state = 0;
	v->buffer[channel].frame[idx].ts = 0;
	v->buffer[channel].frame[idx].cur_size = 0;
	return 0;
}

int S2255CORE_query_buf(HANDLE board, ULONG chn, ULONG idx)
{
    PVPAR v = (PVPAR ) board;
    ULONG channel;
	if (v == NULL)
		return -1;
    if( chn >= MAX_CHANNELS) {
        return -1;
    }
    channel = G_chnmap[chn];
    if( idx >= SYS_FRAMES) {
        return -1;
    }
    return v->buffer[channel].frame[idx].read_state;
}

int S2255CORE_query_buf_ext(HANDLE board, ULONG chn, ULONG idx, UINT32 *ts)
{
    PVPAR v = (PVPAR ) board;
    ULONG channel;
	if (v == NULL)
		return -1;
    if( chn >= MAX_CHANNELS) {
        return -1;
    }
    channel = G_chnmap[chn];
    if( idx >= SYS_FRAMES) {
        return -1;
    }
    if (ts)
        *ts = v->buffer[channel].frame[idx].ts;
    return v->buffer[channel].frame[idx].read_state;
}


#define dprintk(level, fmt, arg...) {} 
// { printk(fmt,##arg);};

static int buffer_to_user( PVPAR v, S2255_USB_PIPE_INFO* pipe_info)
{
	// spinlock acquire when this function called
	char *pdest;
	UINT32 offset = 0;
	char *psrc;
	ULONG copy_size;
	ULONG size;
	INT32 idx = -1;
	FRAMEI *frm;
	int bframe = 0;
	unsigned char *pdata;
	if (v == NULL)
		return -1;
	if (v->is_shutdown)
		return -1;
	idx = v->cur_frame[v->cc];
	frm = &v->buffer[v->cc].frame[idx];
	if (frm->read_state == 0 || v->b_searching) {
		int jj;
		unsigned int cc;
		s32 *pdword;
		int payload;
		/* search for marker codes */
		pdata = (unsigned char *)pipe_info->pTransferBuffer;
		for (jj = 0; jj< pipe_info->cur_transfer_size-12; jj++) {
			switch (*(s32 *) pdata) {
			case S2255_MARKER_FRAME:
				pdword = (s32 *)pdata;
				dprintk(4, "found frame marker at offset:"
					" %d [%x %x]\n", jj, pdata[0],
					pdata[1]);
				offset = jj + PREFIX_SIZE;
				bframe = 1;
				cc = pdword[1];
				if (cc >= MAX_CHANNELS) {
					printk(KERN_ERR
					       "bad channel\n");
					return -EINVAL;
				}
				v->cc = cc;
				payload =  pdword[3];
				if (payload > v->req_image_size[v->cc]) {
					v->bad_payload[v->cc]++;
					printk("bad payload\n");
					/* discard the bad frame */
					return -EINVAL;
				}
				v->pkt_size[v->cc] = payload;
				v->jpg_size[v->cc] = pdword[4];
				v->ts[v->cc] = pdword[5];
				v->cur_color[v->cc] = pdword[2];
				dprintk(4, "channel is %d color %d\n", v->cc, pdword[2]);
				v->b_searching = 0;
				break;
			case S2255_MARKER_RESPONSE:
				pdword = (s32 *)pdata;
				pdata += DEF_USB_BLOCK;
				jj += DEF_USB_BLOCK;
				if (pdword[1] >= MAX_CHANNELS)
					break;
				cc = pdword[1];
				if (!(cc >= 0 && cc < MAX_CHANNELS))
					break;
				switch (pdword[2]) {
				case 0x01:
					/* check if channel valid */
					/* set mode ready */
					dprintk(2, "set mode wake %d\n",cc);
					v->setmode_ready[cc] = 1;
					s2255_set_setmode(v->pdx, cc);
					dprintk(5, "setmode ready %d\n", cc);
					break;
				case 0x10:
					v->chn_ready |= (1 << cc);
					if ((v->chn_ready & 0x0f) != 0x0f)
						break;
					/* all channels ready */
					printk(KERN_INFO "s2255: fw loaded\n");
					s2255_set_fwready(v->pdx);
					break;
				case 0x20:
					v->vid_status[cc] = pdword[3];
					v->dsp_fw = pdword[4];
					dprintk(2, "got vid status %x %d\n",
						pdword[3], cc);
					s2255_set_vidstatus(v->pdx, cc);
					break;
				default:
					printk(KERN_INFO "s2255 unknwn resp\n");
					break;
				}
				pdata++;
				break;
			default:
				pdata++;
				break;
			}
			if (bframe)
				break;
		} /* for */
		if (!bframe)
			return -EINVAL;
	}

	idx = v->cur_frame[v->cc];
	frm = &v->buffer[v->cc].frame[idx];

	/* search done.  now find out if should be acquiring on this channel */
	if (!v->b_acquire[v->cc]) {
		/* we found a frame, but this channel is turned off */
		frm->read_state = 0;
		frm->ts = 0;
		printk("not acquiring\n");
		return -EINVAL;
	}

	if (frm->read_state == 0) {
		frm->read_state = 1;
		frm->cur_size = 0;
	} else if (frm->read_state == 2) {
		v->b_searching = 1;
		return -EINVAL;
	}
	/* skip the marker 512 bytes (and offset if out of sync) */
	psrc = (u8 *)pipe_info->pTransferBuffer + offset;
	if (v->buffer[v->cc].frame[idx].lock_buf == NULL) {
		dbgprint2("S2255: refusing to copy to null user buffer, chn %d, idx %d\n",v->cc, idx);
		frm->read_state = 0;
		frm->ts = 0;
		v->b_searching = 1;
		return -4;
	}
	pdest = frm->lock_buf + frm->cur_size;

	copy_size = (pipe_info->cur_transfer_size - offset);

	size = v->pkt_size[v->cc] - PREFIX_SIZE;

	/* sanity check on pdest */
	if ((copy_size + frm->cur_size) < v->req_image_size[v->cc]) {
		s2255_memcpy(pdest, psrc, copy_size);
	}

	frm->cur_size += copy_size;
	dprintk(4, "cur_size size %lu size %lu \n", frm->cur_size, size);

	if (frm->cur_size >= size) {
		u32 cc = v->cc;
		dprintk(2, "****************[%d]Buffer[%d]full*************\n",
			cc, idx);
		frm->read_state = 2;
		frm->ts = v->ts[cc];
		v->last_frame[cc] = v->cur_frame[cc];
		v->cur_frame[cc]++;
		/* end of system frame ring buffer, start at zero */
		if ((v->cur_frame[cc] == SYS_FRAMES) ||
		    (v->cur_frame[cc] == v->buffer[cc].dwFrames))
			v->cur_frame[cc] = 0;

		v->frame_ready = 1;

		if (v->hsema[cc] ) {
			if (v->mode[cc].single == 1) {
				v->b_acquire[cc] = 0;
			}
		}
		// buffer full, get next buffer
		v->b_searching = 1;
		// search for next available buffer
		v->last_size[cc] = v->jpg_size[cc];
		v->last_ts[cc] = v->ts[cc];

		s2255_wakeup( v->pdx, cc);
		v->frame_count++;

	}
	return 0;
}


void S2255CORE_read_video_callback(HANDLE board, S2255_USB_PIPE_INFO* pipe_info)
{
	PVPAR v = (PVPAR ) board;
	if (v->cc >= MAX_CHANNELS) {  //lint !e774 ( channel will change)
		dbgprint0("invalid channel\n");
		v->cc = 0;
	}
	(void) buffer_to_user( v, pipe_info);
	return;
}

// acquires into frames free buffer of buffer struct
// user must call set_buffer first
int S2255CORE_start_acquire( HANDLE board, ULONG chn)
{
	unsigned char *pBuf;
	int res;
	ULONG channel;
	PVPAR v = (PVPAR ) board;
    unsigned long flags;
	if (v == NULL)
		return -1;

	if (chn >= MAX_CHANNELS) {
		errprint1( "start acquire failed, bad channel %lu\n", chn);
		return -1;
	}
	channel = G_chnmap[chn];

	//dbgprint1( "S2255: start acquire %lu \n", channel);
    spin_lock_irqsave(&v->slock, flags);
    
	if (v->b_acquire[channel] == 0) {
#if DBG
		dbgprint2("setting acquire state %lu %d\n",channel, v->b_acquire[channel]);
#endif
		v->last_frame[channel] = -1;
		v->bad_payload[channel] = 0;
		v->b_acquire[channel] = 1;
		v->cur_frame[channel] = 0;
		// single mode.  could have left the buffer without DQ
		if( v->mode[channel].single) {
			v->buffer[channel].frame[0].read_state = 0;
			v->buffer[channel].frame[0].cur_size = 0;
		}
	} else {
		dbgprint0("already acquiring\n");
        spin_unlock_irqrestore(&v->slock, flags);
		return -2;
	}
	
	// set flag to start acquisition for pipe
	spin_unlock_irqrestore(&v->slock, flags);

	s2255_lock_command(v->pdx);
	pBuf = (unsigned char *) v->cmdbuf;
	// send the start command
	*(UINT32 *)pBuf = IN_DATA_TOKEN;             //lint !e826
	*((UINT32 *) pBuf + 1) = (UINT32) channel;   //lint !e826
	*((UINT32 *) pBuf + 2) = (UINT32) CMD_START; //lint !e826
	res = s2255_write_config( v->pdx, pBuf, 512);
	if (res != 0 ) {
		errprint0("S2255: CMD_START error\n");
	}
	s2255_unlock_command(v->pdx);

#if DBG
	dbgprint1( "start acquire exit[%lu] \n", channel);
#endif
	return 0;
}

int S2255CORE_stop_acquire( HANDLE board, ULONG chn)
{
	unsigned char *pBuf;
	PVPAR v = (PVPAR ) board;
	int i;
	int b_acq = 0;
	int res;
	ULONG channel;
	int j;
    unsigned long flags;
	if (v == NULL)
		return -1;
	if (chn >= MAX_CHANNELS) {
		errprint1( "stop acquire failed, bad channel %lu\n", chn);
		return -1; 
	}
	channel = G_chnmap[chn];
    spin_lock_irqsave(&v->slock, flags);
	if ((v->b_acquire[channel] != 1) && (v->mode[channel].single == 0)) {
		//dbgprint0("already not acquiring\n");
        spin_unlock_irqrestore(&v->slock, flags);
		return -2;
	}
	spin_unlock_irqrestore(&v->slock, flags);

	for(  j=0;j<MAX_CHANNELS;j++) {
		if (v->b_acquire[j] ) {
			b_acq = 1;
			break;
		}
	}
	// stop reading from specified channel
	s2255_lock_command(v->pdx);
	pBuf = (unsigned char *) v->cmdbuf;
	// send the stop command
	*(UINT32 *)pBuf = IN_DATA_TOKEN;        //lint !e826
	*((UINT32 *) pBuf + 1) = (UINT32) channel;  //lint !e826
	*((UINT32 *) pBuf + 2) = CMD_STOP;      //lint !e826
	res = s2255_write_config( v->pdx, pBuf, 512);
	if (res!=0 ) {
		errprint0("CMD_STOP error\n");
	}
	s2255_unlock_command(v->pdx);

	// acquire spinlock
	spin_lock_irqsave(&v->slock, flags);
	v->b_acquire[channel] = 0;
	if (channel == v->cc) {
#if DBG
		dbgprint0("setting search again\n");
#endif
		v->b_searching = 1; 
	}

	spin_unlock_irqrestore(&v->slock, flags);
	// count down the semaphore
	res = s2255_reset_frames( v->pdx, channel);
    
	if (!b_acq) {
		v->b_searching = 1;
	}

	// release all buffers for this channel
	for( i=0;i<SYS_FRAMES;i++) {
		spin_lock_irqsave(&v->slock, flags);
		if (v->buffer[channel].frame[i].pMDL) {
			spin_unlock_irqrestore(&v->slock, flags);
#if DBG
			dbgprint1("unlocking buffers! %p\n",  v->buffer[channel].frame[i].pMDL);
#endif
			s2255_unlock_buffer(  v->buffer[channel].frame[i].pMDL);
			spin_lock_irqsave(&v->slock, flags);

			v->buffer[channel].frame[i].lock_buf = NULL;
			v->buffer[channel].frame[i].pMDL = NULL;
		}
		v->buffer[channel].frame[i].read_state = 0;
		v->buffer[channel].frame[i].cur_size = 0;
		spin_unlock_irqrestore(&v->slock, flags);
	} 
	return 0;
}


int S2255CORE_set_mode( HANDLE board, ULONG chn, MODE2255I *mode)
{
	int res;
	int j;
	// change the mode
	UINT32 *pBuf;
	ULONG channel;
	S2255_USB_PIPE_INFO* pipe_info;
	PVPAR v = (PVPAR ) board;
    unsigned long flags;
	if (chn >= MAX_CHANNELS)
		return -1;
	if (v == NULL)
		return -1;
	channel = G_chnmap[chn];
	spin_lock_irqsave(&v->slock, flags);
	// save the mode
	v->mode[channel] = *mode;
	//dbgprint3("mode scale [%ld] %p %d\n", channel, &v->mode[channel], v->mode[channel].scale);
	if( v->is_open == 0) {
		spin_unlock_irqrestore(&v->slock, flags);
		return -1;
	}

	if (mode->restart) {
		for( j=0;j<MAX_PIPE_BUFFERS;j++) {
			pipe_info = &v->UsbPipes[INDEX_READ_PIPE][j];
			pipe_info->cur_transfer_size = DEF_USB_BLOCK; //(UINT32) mode->usb_block;
			mode->usb_block = DEF_USB_BLOCK;
#if DBG
			dbgprint1("setting transfer size %d \n", mode->usb_block);
#endif
			if (pipe_info->cur_transfer_size > pipe_info->max_transfer_size) {
#if DBG
				dbgprint0("pipe size too big!\n");
#endif
				pipe_info->cur_transfer_size = pipe_info->max_transfer_size;
			}
		}
		v->req_image_size[channel] = get_transfer_size( mode);
	}

	spin_unlock_irqrestore(&v->slock, flags);

	s2255_lock_command(v->pdx);
	pBuf = (UINT32 *) v->cmdbuf;
	s2255_memset( pBuf, 512,0);
	// set the mode
	pBuf[0] = IN_DATA_TOKEN;
	pBuf[1] = (UINT32) channel;
	pBuf[2] = CMD_SET_MODE;
	s2255_memcpy( pBuf + 3, mode, sizeof( MODE2255I));

	res = s2255_write_config( v->pdx,(UINT8 *) pBuf,512);
	if (mode->restart) {
		// wait for response
		res = s2255_wait_setmode(v->pdx, chn);
	}
	s2255_unlock_command(v->pdx);
	return res;
}


ULONG S2255CORE_get_busy_buffers( HANDLE board, ULONG chn)
{
	PVPAR v = (PVPAR ) board;
	ULONG idx;
	ULONG count = 0;
	ULONG channel;
	if (chn >= MAX_CHANNELS)
		return 0;
	if (v == NULL)
		return -1;
	channel = G_chnmap[chn];
	for( idx = 0 ; idx < v->buffer[channel].dwFrames; idx++) {
		if( v->buffer[channel].frame[idx].read_state == 2)
			count++;
	}
	return count;
}

ULONG S2255CORE_get_working_buffers( HANDLE board, ULONG chn)
{
	PVPAR v = (PVPAR ) board;
	ULONG idx;
	ULONG count = 0;
	ULONG channel;
	if (v == NULL)
		return -1;
	if( chn >= MAX_CHANNELS)
		return 0;
	channel = G_chnmap[chn];
	for( idx = 0 ; idx < v->buffer[channel].dwFrames; idx++) {
		if( v->buffer[channel].frame[idx].read_state == 1)
			count++;
	}
	return count;
}


ULONG S2255CORE_get_frame_count( HANDLE board)
{
	PVPAR v = (PVPAR ) board;
	if (v == NULL)
		return -1;
	return v->frame_count;
}


int S2255CORE_get_frame_ready( HANDLE board, int **ppFR)
{
	PVPAR v = (PVPAR ) board;
	if (v == NULL)
		return -1;
	*ppFR = &v->frame_ready;
	//dbgprint1("buffer ready: FR addr %p\n",&v->frame_ready);
	return 0;
}


int S2255CORE_get_mode( HANDLE board, ULONG chn, MODE2255I *mode)
{
	ULONG channel;
	PVPAR v = (PVPAR ) board;
	if (chn >= MAX_CHANNELS)
		return -1;
	if (v == NULL)
		return -1;
	channel = G_chnmap[chn];
	*mode = v->mode[channel];
#ifdef DEBUG_VERBOSE
	dbgprint0("-------------------------------------------------------\n");
	dbgprint1("GET MODE: channel %d\n", channel);
	dbgprint1("format: %d\n", mode->format);
	dbgprint1("scale: %d\n", mode->scale);
	dbgprint1("fdec: %d\n", mode->fdec);
	dbgprint1("color: %d\n", mode->color);
	dbgprint3("mode scale [%d] %p %d\n", channel, mode, mode->scale);
	dbgprint3("mode scale [%d] %p %d\n", channel, &v->mode[channel], v->mode[channel].scale);
	dbgprint0("-------------------------------------------------------\n");
#endif
	return 0;
}


// Linux.  Create the buffers beforehand to copy out of
// we could just do read and then do this from userspace,
// but it is not as compatible or flexible
int S2255CORE_create_sys_buffers( HANDLE board,ULONG chn)
{
	PVPAR v = (PVPAR ) board;
	ULONG i;
	ULONG reqsize;
	ULONG channel ;
    unsigned long flags;
#if DBG
	dbgprint2( "setting channel %lu, buffer %p\n", channel, pbuf);
#endif
	if (chn >= MAX_CHANNELS) {
		return -1;
	}
	channel = G_chnmap[chn];
	spin_lock_irqsave(&v->slock, flags);
	// don't allow buffer registration if stream is currently acquiring.
	if (v->b_acquire[channel] ) {
		dbgprint0("S2255: stop the acquisition first\n");
		spin_unlock_irqrestore(&v->slock, flags);
		return -1;
	}
	if (v->buffers_reg[channel] ) {
#if DBG
		dbgprint0("warning buffers already registered!\n");
#endif
	}
	v->buffer[channel].dwFrames = SYS_FRAMES;
	// always allocate maximum size(PAL)
	reqsize = SYS_FRAMES_MAXSIZE;
	spin_unlock_irqrestore(&v->slock, flags);
	for( i= 0;i< SYS_FRAMES;i++) {
		// allocate the frames
		v->buffer[channel].frame[i].lpvbits = s2255_alloc_mem( reqsize);
		if (v->buffer[channel].frame[i].lpvbits == NULL) {
			dbgprint0( "Out of vmalloc space.  increase it in your boot kernel boot parameters\n");
		}
		v->buffer[channel].frame[i].size = reqsize;
	}
#if DBG
	dbgprint1( "buffer %p\n", v->buffer[channel]->frame[0].lpvbits);
#endif
	spin_lock_irqsave(&v->slock, flags);
	// make sure internal states are set
	for(i=0;i<SYS_FRAMES;i++) {
		// these are sys frames.  No need to lock them(for copying direct to user space.  Just set the lock buf the same as the allocated buf above        
		v->buffer[channel].frame[i].lock_buf = v->buffer[channel].frame[i].lpvbits;
		v->buffer[channel].frame[i].pMDL = NULL;
		v->buffer[channel].frame[i].read_state = 0;
		v->buffer[channel].frame[i].cur_size = 0;
	}

	v->buffers_reg[channel] = 1;
	v->cur_frame[channel] = 0;
	v->last_frame[channel] = -1;
	spin_unlock_irqrestore(&v->slock, flags);

	return v->map_failed ? -1 : 0;
}

int S2255CORE_release_sys_buffers( HANDLE board, ULONG channel )
{
	PVPAR v = (PVPAR ) board;
	ULONG i;

	for( i= 0;i< SYS_FRAMES;i++) {
		if (v->buffer[channel].frame[i].lpvbits ) {
			s2255_free_mem( v->buffer[channel].frame[i].lpvbits);
		}
		v->buffer[channel].frame[i].lpvbits = NULL;
		v->buffer[channel].frame[i].lock_buf = NULL;
	}
	return 0;
}

// for Linux, to copy from sys buffer to user
int S2255CORE_copy_buffer( HANDLE board, ULONG chn, ULONG frmnum, char *pbuf, ULONG size)
{
	int channel;
	char *psrc;

	PVPAR v = (PVPAR ) board;
	if (v == NULL)
		return -1;
	if (chn >= MAX_CHANNELS) {
		dbgprint0("copybuffer:wrong channel!\n");
		return -1;
	}
	channel = G_chnmap[chn];
	if (v->buffer[channel].frame[frmnum].read_state != 2) {
		dbgprint2("buffer NOT ready channel %d, frmnum %ld!\n", channel, frmnum);
		return -2;
	}
	if (size > v->buffer[channel].frame[frmnum].size) {
		size = v->buffer[channel].frame[frmnum].size;
	}
	psrc = v->buffer[channel].frame[frmnum].lpvbits;
	//    printk("copy buffer %ld \n", size);
	s2255_copy_kernel_to_user( pbuf, psrc, size);
	return 0;
}


int S2255CORE_getlastframe( HANDLE board, ULONG chn, int *last_frame, int *last_size, unsigned int *last_ts)
{
	int channel;

	PVPAR v = (PVPAR ) board;
	if (chn >= MAX_CHANNELS) {
		dbgprint0("wrong channel!\n");
		return -1;
	}
	channel = G_chnmap[chn];
	*last_frame = v->last_frame[channel];
	*last_size = v->last_size[channel];
	if (last_ts)
		*last_ts = v->last_ts[channel];
	v->last_frame[channel] = -1;
	v->last_ts[channel] = -1;
	return 0;
}

void * S2255CORE_get_buffer( HANDLE board, ULONG chn, ULONG frmnum )
{
	int channel;
	PVPAR v = (PVPAR ) board;
	if (v == NULL) {
		dbgprint0("get buffer error\n");
		return NULL;
	}
	if (chn >= MAX_CHANNELS) {
		dbgprint0("wrong channel!\n");
		return NULL;
	}
	channel = G_chnmap[chn];
	return v->buffer[channel].frame[frmnum].lpvbits;
}

#if defined (DRIVER_DSHOW)
void S2255CORE_reg_pin( HANDLE board, ULONG chn, PVOID pin)
{
	int channel;
	PVPAR v = (PVPAR ) board;
	if (v == NULL) {
		return;
	}
	channel = G_chnmap[chn];

	v->pin[channel] = pin;
}
#endif

int S2255CORE_get_status(HANDLE board, int chn, UINT32 *pvidstatus)
{
	unsigned char *pBuf;
	int res;
	ULONG channel;
	PVPAR v = (PVPAR ) board;	
	if (v == NULL)
		return -1;
	if (chn >= MAX_CHANNELS) {
		errprint1( "start acquire failed, bad channel %d\n", chn);
		return -1;
	}
	channel = G_chnmap[chn];
	s2255_lock_command(v->pdx);
	pBuf = (unsigned char *) v->cmdbuf;

	/* send the start command */
	*(UINT32 *)pBuf = IN_DATA_TOKEN;             //lint !e826
	*((UINT32 *) pBuf + 1) = (UINT32) channel;   //lint !e826
	*((UINT32 *) pBuf + 2) = (UINT32) CMD_STATUS; //lint !e826
	

	res = s2255_write_config( v->pdx, pBuf, 512);
	if (res != 0 ) {
		errprint0("S2255: CMD_START error\n");
	}
	s2255_unlock_command(v->pdx);

	if ((s2255_wait_vid_status(v->pdx, chn) == 0) && (res == 0)) {
		*pvidstatus = v->vid_status[channel];
//		printk("channel %ld %x\n", channel, *pvidstatus);
		return 0;
	} else
		*pvidstatus = 0;
	return -1;
}


int S2255CORE_get_dspfw(HANDLE board, UINT32 *fw)
{
	unsigned char *pBuf;
	int res;
	ULONG channel;
	PVPAR v = (PVPAR ) board;	
	channel = G_chnmap[0];

    s2255_lock_command(v->pdx);
	pBuf = (unsigned char *) v->cmdbuf;

	/* send the start command */

	*(UINT32 *)pBuf = IN_DATA_TOKEN;             //lint !e826
	*((UINT32 *) pBuf + 1) = (UINT32) channel;   //lint !e826
	*((UINT32 *) pBuf + 2) = (UINT32) CMD_STATUS; //lint !e826
	
	res = s2255_write_config( v->pdx, pBuf, 512);
	if (res != 0 ) {
		errprint0("S2255: CMD_START error\n");
	}
    s2255_unlock_command(v->pdx);

	if ((s2255_wait_vid_status(v->pdx, 0) == 0) && (res == 0)) {
		*fw = v->dsp_fw;
//		dprintk(2, "firmware %d\n", *fw);
		return 0;
	} else
		*fw = 0;
	return -1;
}

