#ifndef _S2255CORE_H
#define _S2255CORE_H

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
#include <linux/config.h>
#endif
#include <linux/module.h>
#include <linux/mman.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kref.h>
#include <linux/vmalloc.h>
#include <asm/uaccess.h>
#include <linux/usb.h>
#include <linux/dma-mapping.h>
#include <linux/highmem.h>
#include <linux/kthread.h>
#include "s2255_i.h"

#define NUM_ATTR_IDS 32
#define SYS_FRAMES_MAXSIZE     (576*720*3+4096)
#if defined (OSTYPE_WINDOWS)
#define MAX_PIPE_USBBLOCK     (128*1024)
#elif defined (OS_LINUX)
#define MAX_PIPE_USBBLOCK     (40*1024)
#endif


#define DEF_USB_BLOCK (16*1024)
#define MAX_PIPE_NUMBER 2
#define MAX_CHANNELS    4

struct _S2255_USB_PIPE_INFO;
/* stream call back function */
typedef void (* STREAM_CALLBACK)(HANDLE board, struct _S2255_USB_PIPE_INFO *pPipe);
/* stream thead mode procedure */
typedef void (* STREAM_THREADPROC)( HANDLE board);

typedef struct _S2255_USB_PIPE_INFO
{
	UINT32				cur_transfer_size;
	UINT32				max_transfer_size;
	UINT8*				pTransferBuffer;
	UINT32				dwTransferFlags;
	UINT32				dwState;
	UINT32              dwPrevState;
	UINT32				dwUrbSize;
	PVOID           	hPipeHandle;
	PVOID           	stream_urb;
	HANDLE				hThreadHandle;
	STREAM_CALLBACK		pCallBack;
	STREAM_THREADPROC	pThreadProc;
	PVOID               pdx;
    HANDLE              hbd;
    UINT32              dwErrorCount;
    UINT32              dwBufIdx;
    PVOID                rcitem; // work item for recovery
    PVOID                Irp; // Irp we are going to fail
    int                 priority_set;
    UINT32              idx;
    wait_queue_head_t   hStopEvent;

} S2255_USB_PIPE_INFO;

#if defined (OS_LINUX) 
#define MAX_PIPE_BUFFERS 1
#elif defined (OSTYPE_WINDOWS)
#define MAX_PIPE_BUFFERS 4
#endif

typedef struct {
	PVOID               dbgaddr; // for compile verification
    ULONG			    m_attr[NUM_ATTR_IDS];	// board parameters.  declared here for now
    int                 board_num;
    int                 product_id;
    int                 is_open;
    int                 is_shutdown;
    ULONG               debug_flags;
	spinlock_t          slock;
    PVOID               cmdbuf;
    PVOID               pdx;
    HANDLE              hbd;
    S2255_USB_PIPE_INFO UsbPipes[MAX_PIPE_NUMBER][MAX_PIPE_BUFFERS];
    ULONG               buf_size;
    PVOID               pBuffer;
    BUFFERI             buffer[MAX_CHANNELS];
    //PVOID               hevent[MAX_CHANNELS];
    PVOID               hsema[MAX_CHANNELS];
    MODE2255I           mode[MAX_CHANNELS];
    int                 cur_frame[MAX_CHANNELS];
    int                 last_frame[MAX_CHANNELS];
    int                 last_size[MAX_CHANNELS];
    int                 map_failed;
    int                 b_acquire[MAX_CHANNELS];
    int                 b_acquire_test;
    UINT32              acq_test_size;
    int                 acq_num_bytes;
    //UINT8               *work_image[MAX_CHANNELS];  // work buf
    UINT32                cc; // current channel
    int                 b_searching;
    ULONG               req_image_size[MAX_CHANNELS];
    ULONG               pkt_size[MAX_CHANNELS];
    ULONG               jpg_size[MAX_CHANNELS];
    ULONG               ts[MAX_CHANNELS];
    ULONG               last_ts[MAX_CHANNELS];
    ULONG               cur_color[MAX_CHANNELS];
    int                 bad_payload[MAX_CHANNELS];
    int                 buffers_reg[MAX_CHANNELS];
    UINT32              frame_count;
	int                 frame_ready;

#if defined (DRIVER_DSHOW)
    // callback to the pin
    PVOID               pin[MAX_CHANNELS]; 
#endif
	int setmode_ready[MAX_CHANNELS];
	int vidstatus_ready[MAX_CHANNELS];
	int chn_ready;
	int vid_status[MAX_CHANNELS];
	UINT32 fwver;
  	int dsp_fw;
}VPAR, *PVPAR;

#ifdef __cplusplus
extern "C" {
#endif


PVOID S2255CORE_board_init( int product_id, PVOID pdx);

/** initializes the I2C settings.
 *  files are DEC_xy.ini, ENC_xy.ini, tlv320.ini.
 *  File are in the main system directory.
 *  These are hardcoded here for now, but will be moved to the registry later
 *  This should be cleaned up in Rev 2.   
 *  1) TLV does not need set each time decoder/encoder set
 *  2) dec_, enc_ ini settings could be moved to the DLL where SDI inputs are also set.  
 *          Right now, architecture is not very consistent.  Or all settings moved to driver.
 *          The separation right now is because driver was going to be ported to Linux, but ran out of time
 *  @param pdx pointer to device extension
 *  @param mask 1==decoder, 2==encoder, 4==tlv audio (see board.h for enumeration)
 *  @return 0 on success
 *  
 */
int S2255CORE_init_settings(HANDLE board, int mask);

/** reloads the firmware
 *  @param pdx pointer to device extension
 *  @return 0
 */
int S2255CORE_reset_firmware( HANDLE board);

/** write burst of data to i2c device
 *  @param pdx pointer to device extension
 *  @param addr address of I2C device to write to
 *  @param length of data to write
 *  @param data pointer to data to write( should include subaddress if necessary)
 *  @return I2C_WriteReg2 result
 */
int S2255CORE_write_i2c_burst( HANDLE board, UINT8 addr, UINT8 length, UINT8 *data);
int S2255CORE_write_i2c_burst_eeprom( HANDLE board, UINT8 addr, UINT8 length, UINT8 *data);


/** write single byte to i2c device
 *  @param pdx pointer to device extension
 *  @param addr address of I2C device to write to
 *  @param subaddr subaddress to write to
 *  @param value value to write
 *  @return I2C_WriteRegister result
 */
int S2255CORE_write_i2c( HANDLE board, UINT8 addr, UINT16 subaddr, UINT8 value);

/** read single byte from I2C
 *  @param pdx pointer to device extension
 *  @param addr address of I2C device to read
 *  @param subaddr subaddress to read from
 *  @return value read from I2C (or 0xcd if invalid)
 */
int S2255CORE_read_i2c( HANDLE board, UINT8 addr, UINT16 subaddr);

/** read I2c without specify a subaddress, aka burst read.
 *  @param pdx pointer to device extension
 *  @param addr I2c main address
 *  @param datalen data length to read
 *  @param data pointer to data
 *  @return I2C_ReadReg2 result
 */

int S2255CORE_read_i2c_nostart( HANDLE board, UINT8 addr, UINT8 datalen, UINT8 *data);

int S2255CORE_read_i2c_burst( HANDLE board, UINT8 addr, UINT16 subaddr, UINT8 *data, UINT8 datalen);



/** get the video status from the analog video decoder (ie. 7115)
 *  @param pdx pointer to device extension
 *  @return video status returned from the decoder
 */
char S2255CORE_in_get_status( HANDLE board);


/** write address before a block transfer
 *  @param pdx pointer to device extension
 *  @param addr address to begin block transfer to
 *  @return 0 on success
 */
int S2255CORE_write_address( HANDLE board, UINT8 addr );


int S2255CORE_write_burstdata( HANDLE board, UINT8 *data, ULONG size);

/** write one data byte to an address
 *  @param pdx device extension
 *  @param addr address
 *  @param data data
 *  @return 0 on success
 */
int S2255CORE_write_addrdata_single( HANDLE board, UINT8 addr, UINT8 data);

/** writes to the stratix using continuous address/data stream
 *  @param data - pointer to data  eg.  data[0]=0xf0, data[1]= 3; data[2]=0x4c, data[3]=5, etc...
 *                writes 3 to address 0xf0, and writes 5 to address 0x4c
 *  @param size - length of data(should be multiple of 2! see above)                
 *  @return 0 on success
 */
int S2255CORE_write_addrdata_fifo( HANDLE board, UINT8 *data, ULONG size);


/** sends a reset pulse to the FPGA
 *  @param pdx pointer to device extension
 *  @return 0 on success
 */
int S2255CORE_reset_stratix( HANDLE board);

/** Stratix reset line on
 *  @param pdx pointer to device extension
 *  @return 0 on success
 */
int S2255CORE_reset_stratix_on( HANDLE board);

/** take reset line off
 *  @param pdx pointer to device extension
 *  @return 0 on success
 */
int S2255CORE_reset_stratix_off( HANDLE board);


/** reads a single byte from address specified
 *  @param addr- address to read
 *  @param *outbuf- pointer to result
 *  @return 0 on success 
 */
int S2255CORE_read_stratix( HANDLE board, UINT8 addr, UINT8 *outbuf);

/** sets up a FIFO read from the stratix
 *  @param addr -- address to read
 *  @param blocks -- read size in blocks(512 bytes)
 *  @return 0 on success
 */

int S2255CORE_read_fifo( HANDLE board, UINT8 addr, UINT16 blocks);

/** set the read size.  obsoleted.
 * @param pdx pointer to device extension
 * @param size size of read
 * @return 0
 */
int S2255CORE_set_readsize( HANDLE board, UINT8 size);


/** set attributes in the driver.  Currently, defined attributes are video type( Composite/SVideo) and
  * video system (NTSC/PAL).  In the future more attributes may be moved from the middleware(DLL) to the driver
  * @param pdx pointer to device extension
  * @param attr attribute to set( ATTR_video_system or ATTR_video_type)
  * @return 0 on success
  */

int S2255CORE_attr_set( HANDLE board, ULONG attr, ULONG val);

/** retrieve FX2 firmware version
 * @param pdx pointer to device extension
 * @return -1 for fail, else returns firmware version as an int(16 bits)
 */
int S2255CORE_get_fx2fw( HANDLE board);

/** write to PORT C on the Stratix 
 *  @param pdx pointer to device extension
 *  @param data data to write to port C
 *  @return value returned from firmware
 */
int S2255CORE_write_portc( HANDLE board, UINT8 data);

int S2255CORE_start_acquire( HANDLE board, ULONG channel );
int S2255CORE_stop_acquire( HANDLE board, ULONG channel );


int S2255CORE_set_mode( HANDLE board, ULONG chan, MODE2255I *mode);
int S2255CORE_get_mode( HANDLE board, ULONG chan, MODE2255I *mode);

int S2255CORE_set_buffer_dbg( HANDLE board, PVOID pbuf, ULONG size);

int S2255CORE_board_open( HANDLE board);
int S2255CORE_board_close( HANDLE board);
int S2255CORE_board_shutdown( HANDLE board);
int S2255CORE_dqbuf( HANDLE board, ULONG chan, ULONG idx);
void S2255CORE_read_video_callback(HANDLE context, S2255_USB_PIPE_INFO *pPipe);

int S2255CORE_start_acquire_test( HANDLE board, ULONG xfersize );
int S2255CORE_stop_acquire_test( HANDLE board );

ULONG S2255CORE_get_busy_buffers( HANDLE board, ULONG chan);
int S2255CORE_get_events_sent( HANDLE board);
ULONG S2255CORE_get_frame_count( HANDLE board);
int S2255CORE_get_frame_ready( HANDLE board, int **ppFR);
int S2255CORE_create_sys_buffers( HANDLE board, ULONG chn);
int S2255CORE_release_sys_buffers( HANDLE board, ULONG chn);
int S2255CORE_copy_buffer( HANDLE board, ULONG chn, ULONG frmnum, char *pbuf, ULONG size);
	int S2255CORE_getlastframe( HANDLE board, ULONG chn, int *last_frame,
                                int *last_size, unsigned int *last_ts);
void * S2255CORE_get_buffer( HANDLE board, ULONG chn, ULONG idx );
int S2255CORE_get_status(HANDLE board, int chn, UINT32 *pvidstatus);
#if defined (DRIVER_DSHOW)
void S2255CORE_reg_pin( HANDLE board, ULONG idx, PVOID pin);
#endif
int S2255CORE_get_dspfw(HANDLE board, UINT32 *fw);

int S2255CORE_query_buf(HANDLE board, ULONG chan, ULONG idx);
int S2255CORE_query_buf_ext(HANDLE board, ULONG chan, ULONG idx, UINT32 *ts);

#ifdef __cplusplus
}
#endif
#endif
