/*** sx2255ioctl.h
 * Copyright(C) Sensoray Company Inc. 2007
 * @author: Dean Anderson
 * 2255 IOCTL commands.  Linux and Windows
 */
#ifndef _S2255IOCTL_H
#define _S2255IOCTL_H

#include "s2255.h"
#include "s2255_i.h"

typedef struct {
    UINT32    idx;
    MODE2255I mode;
} set_mode_param_t;

typedef struct {
    UINT32 idx;
    BUFFER buf;
} set_buffer_param_t;

typedef struct {
    UINT32 idx;
    HANDLE hevent;
} reg_event_param_t;

typedef struct {
    UINT32 chn;
    UINT32 frm;
} dqbuf_param_t;

typedef struct {
    UINT32 idx;
} start_param_t;

typedef struct {
    UINT32 idx;
} stop_param_t;

typedef struct {
    UINT32 idx;
} unreg_event_param_t;

typedef struct {
    UINT32 idx;
    UINT32 last_frame; // last frame processed returned from call if success
    UINT32 timeout;//ms (only for WAIT_BLOCK)
    UINT32 last_size; //size of last frame(for JPEG)
    UINT32 last_ts; //last timestamp in ms (1ms resolution)
} wait_param_t;


typedef struct {
    UINT32 idx;     // channel index
    UINT32 frm_num; // frame number to GET
    UINT32 size;    // size of the buffer
    char   *pbuf;
   UINT32 *psize; // pointer to size in kernel
} getbuf_param_t;

typedef struct {
	UINT32 idx;     // channel index
	UINT32 status; //
} get_vidstatus_t;

typedef struct {
    UINT8 addr;
	UINT16 subaddr;
	UINT8 *data;
	UINT8 datalen;
} get_sn_param_t;

typedef struct {
	UINT32 dsp_fw;   
	UINT32 usb_fw;
} get_firmware_t;

typedef struct {
	UINT32 chn;   // channel 
	UINT32 frm;   // frame
	UINT32 state; // state
} querybuf_t;

typedef struct {
	UINT32 chn;   // channel 
	UINT32 frm;   // frame
	UINT32 state; // state
	UINT32 ts;    // timestamp
} querybuf_ext_t;

typedef struct {
	UINT32 chn;   // channel 
	UINT32 state[SYS_FRAMES]; // state
	UINT32 ts[SYS_FRAMES];    // timestamp
} querybuf_all_t;

#define MAX_READ_XFER	512

#ifdef WIN32
	#define IOC_OUT_DIRECT( id_lin, id_win )	CTL_CODE( FILE_DEVICE_UNKNOWN, id_win, METHOD_OUT_DIRECT, FILE_ANY_ACCESS ) 
	#define IOC_IN_DIRECT(  id_lin, id_win )	CTL_CODE( FILE_DEVICE_UNKNOWN, id_win, METHOD_IN_DIRECT , FILE_ANY_ACCESS ) 
	#define IOC_BUFFERED(   id_lin, id_win )	CTL_CODE( FILE_DEVICE_UNKNOWN, id_win, METHOD_BUFFERED,   FILE_ANY_ACCESS ) 
	#define IOC_NEITHER(    id_lin, id_win )	CTL_CODE( FILE_DEVICE_UNKNOWN, id_win, METHOD_NEITHER,    FILE_ANY_ACCESS ) 
#else    
	#define IOC_OUT_DIRECT( id_lin, id_win )	id_lin 
	#define IOC_IN_DIRECT(  id_lin, id_win )	id_lin 
	#define IOC_BUFFERED(   id_lin, id_win )	id_lin 
	#define IOC_NEITHER(    id_lin, id_win )	id_lin 
#endif

#if defined(OSTYPE_WINDOWS)
// WINDOWS only.  not used in Linux
#define S2255_IOC_REG_EVENT                IOC_IN_DIRECT   ( 0x1000, 0x800 )
#define S2255_IOC_UNREG_EVENT              IOC_IN_DIRECT   ( 0x1002, 0x802 )
#define S2255_IOC_SET_BUFFER               IOC_IN_DIRECT   ( 0x1004, 0x804 ) // set buffer, ULONG-channel, PVOID buffer struct
// internal and future use
#define S2255_IOC_TEST_WRITE			    IOC_OUT_DIRECT  ( 0x1200, 0x900 ) // write data to the fifo(after IOC_WRITE_ADDRESS)
#define S2255_IOC_TEST_READ			    IOC_OUT_DIRECT  ( 0x1202, 0x902 ) // write data to the fifo(after IOC_WRITE_ADDRESS)

#define S2255_IOC_SET_BUFFER_TEST          IOC_IN_DIRECT   ( 0x1208, 0x908 )
#define S2255_IOC_GET_EVENT                IOC_OUT_DIRECT  ( 0x120a, 0x90a )

// for start and stop
#define S2255_IOC_TEST_ACQUIRE_START        IOC_IN_DIRECT  ( 0x1210, 0x910)
#define S2255_IOC_TEST_ACQUIRE_STOP         IOC_IN_DIRECT  ( 0x1212, 0x912)

#define S2255_IOC_GET_BUSY_BUFFERS          IOC_OUT_DIRECT( 0x1214, 0x914)
#define S2255_IOC_GET_CALLBACK_COUNT        IOC_OUT_DIRECT( 0x1216, 0x916)
#define S2255_IOC_GET_EVENT                IOC_OUT_DIRECT  ( 0x120a, 0x90a )
#endif



#define S2255_IOC_SET_MODE                 IOC_IN_DIRECT   ( 0x1006, 0x806 )
#define S2255_IOC_STARTACQUIRE             IOC_IN_DIRECT   ( 0x1008, 0x808 )
#define S2255_IOC_STOPACQUIRE              IOC_IN_DIRECT   ( 0x100a, 0x80a )
#define S2255_IOC_GET_FX2FW				IOC_OUT_DIRECT	( 0x100c, 0x80c )  // retrieve FX2 USB FW
#define SX2255_IOC_READ_I2C_BURST	   IOC_IN_DIRECT   ( 0x13a8, 0xa08 )
#define S2255_IOC_DQBUF                    IOC_IN_DIRECT   ( 0x1010, 0x810 )
#if !defined(OSTYPE_WINDOWS)
#define S2255_IOC_WAIT_FRAME_NONBLOCK       IOC_OUT_DIRECT( 0x1316, 0x936)
#define S2255_IOC_WAIT_FRAME_BLOCK          IOC_OUT_DIRECT( 0x1318, 0x938)
#endif



// for double-buffer(inefficient)
#define S2255_IOC_COPY_BUFFER               IOC_IN_DIRECT  ( 0x1324, 0x944 )
// for MMAP
#define S2255_IOC_REQBUF                    IOC_IN_DIRECT  ( 0x1326, 0x946 )

#define S2255_IOC_GET_MODE                 IOC_IN_DIRECT   ( 0x1328, 0x948 )
// debug
#define S2255_IOC_GET_FRAME_COUNT           IOC_OUT_DIRECT( 0x1218, 0x918)

// vid
#define S2255_IOC_GET_STATUS     IOC_OUT_DIRECT( 0x12f8, 0x9f8)

#define S2255_IOC_GET_FIRMWARE   IOC_OUT_DIRECT( 0x12fa, 0x9fa)


// query buffer
#define S2255_IOC_QUERYBUF         0x1300
#define S2255_IOC_QUERYBUF_EXT     0x1302
#define S2255_IOC_QUERYBUF_ALL     0x1304
#endif //_S2255IOCTL_H

