//**********************************************************
//Sensoray frame grabber model 2255 DLL v.0.01
//Copyright (C) 2007 Sensoray
//Dean A.
//Alexander K.
// Windows:
// StartAcquire returns a semaphore app can wait on.
// Semaphore assigned in driver
//
// Linux:
// StartAcquire does not return a semaphore.
// App should call S2255_check_frame() to check if a frame is ready.
// This is done because in Linux can't signal semaphore directly from kernel
// space to user space as in Windows.
// A note on Video for Linux (V4L or V4L2): (see s2255.c)

#include <stdio.h>
#if defined (OSTYPE_WINDOWS)
#include <windows.h>
#include <winioctl.h>
#elif defined (OSTYPE_LINUX)
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <sched.h>
#include <sys/ioctl.h>
#include <semaphore.h>
#else
#error "must define OSTYPE_ type"
#endif

#include "image.h"
#include "s2255_i.h"
#include "s2255.h"
#include "assert.h"
#include "s2255ioctl.h"

#if defined (OSTYPE_WINDOWS)
#define MID2255SPEC __declspec(dllexport)
// platform independant ioctl
#define pi_ioctl(p1,p2,p3,p4,p5,p6,p7,p8) DeviceIoControl(p1,p2,p3,p4,p5,p6,p7,p8)
// ioctl pass if zero
#define ioctl_pass( res) (res == TRUE)

#elif defined (OSTYPE_LINUX)
#define MID2255SPEC
#define WINAPI
// platform independant ioctl
#define pi_ioctl(p1,p2,p3,p4,p5,p6,p7,p8)  ioctl(p1,p2,p3)
// ioctl pass if zero
#define ioctl_pass( res) ( res == 0)
#define OutputDebugString printf
#define CloseHandle close
#endif


#define LINE_SZ_4CIFS_NTSC  640
#define LINE_SZ_2CIFS_NTSC  640
#define LINE_SZ_1CIFS_NTSC  320 

#define LINE_SZ_4CIFS_PAL  704
#define LINE_SZ_2CIFS_PAL  704
#define LINE_SZ_1CIFS_PAL  352


#define NUM_LINES_4CIFS_NTSC 240
#define NUM_LINES_2CIFS_NTSC 240
#define NUM_LINES_1CIFS_NTSC 240

#define NUM_LINES_4CIFS_PAL  288
#define NUM_LINES_2CIFS_PAL  288
#define NUM_LINES_1CIFS_PAL  288

#define LINE_SZ_DEF             640
#define NUM_LINES_DEF           240         //this is per field!

#define MAX_WIDTH     LINE_SZ_4CIFS_PAL
#define MAX_HEIGHT    (NUM_LINES_4CIFS_PAL *2)

#define DEF_USB_BLOCK (4096)
static int get_width( MODE2255 *mode);
static int get_height( MODE2255 *mode);



//#define USB_READ_SIZE     0x00001000      
//depends on USB_READ_SIZE
//#define MASK_MULT         0xfffff000
#define PREFIX_SIZE         512             //frame prefix size (sent once every frame)


/**********************************************************
 * Global variables
 *********************************************************/
//Shared
DWORD g_IFlag = 0;            //initialization flag;

HANDLE g_hMutex = NULL;             //mutex to protect data from simult access

#define MAX_BOARDS 8

typedef struct {
	int m_acquiring[MAX_CHANNELS];// = {0,0,0,0};
	MODE2255 m_mode[MAX_CHANNELS];
	DWORD m_threadid[MAX_CHANNELS];
	BUFFER m_workbuffer[MAX_CHANNELS];
	BUFFER m_userbuffer[MAX_CHANNELS];
	LPBITMAPINFO m_lpbmi[MAX_CHANNELS];// = {NULL, NULL, NULL, NULL};
	HDEVICE hdev;
	unsigned char *m_work_image;// = NULL;
} board_param_t;

static board_param_t G_board[MAX_BOARDS];



#if defined (OSTYPE_WINDOWS)
static HANDLE G_uevents[MAX_CHANNELS] = {NULL,NULL,NULL,NULL};
static HANDLE G_devents[MAX_CHANNELS] = {NULL,NULL,NULL,NULL};
static HANDLE G_kill[MAX_CHANNELS] = {NULL,NULL,NULL,NULL};
static HANDLE G_acq_handle[MAX_CHANNELS] = {NULL,NULL,NULL,NULL};
#endif


int find_board( HDEVICE hdev)
{
	int i;
	for(i=0;i< MAX_BOARDS;i++) {
		if (G_board[i].hdev == hdev)
			return i;
	}
	return -1;
}


#define MUTEXNAME "S2255MUTEXNAMED"

static int check_channel( int chn) 
{
	if (chn < 1)
		return -1;
	if (chn > MAX_CHANNELS)
		return -1;
	return 0;
}

static int copy_mode_to_modei( MODE2255 *pmode, MODE2255I *pmodei)
{
	pmodei->bright = pmode->bright;
	pmodei->color = pmode->color;
	pmodei->contrast = pmode->contrast;
	pmodei->fdec = pmode->fdec;
	pmodei->format = pmode->format;
	pmodei->hue = pmode->hue;
	pmodei->saturation = pmode->saturation;
	pmodei->single = pmode->single;
	pmodei->scale = pmode->scale;
	return 0;
}

static int copy_modei_to_mode( MODE2255I *pmodei, MODE2255 *pmode)
{
	pmode->bright = pmodei->bright;
	pmode->color = pmodei->color;
	pmode->contrast = pmodei->contrast;
	pmode->fdec = pmodei->fdec;
	pmode->format = pmodei->format;
	pmode->hue = pmodei->hue;
	pmode->saturation = pmodei->saturation;
	pmode->single = pmodei->single;
	pmode->scale = pmodei->scale;
	return 0;
}

#ifndef OSTYPE_LINUX
DWORD WINAPI AcquireThread( LPVOID param) 
{
	HANDLE hobj[2];
	DWORD wres;
	int   chn = (int)param;
	ULONG activebuf = 0;
	hobj[0] = G_devents[chn];
	hobj[1] = G_kill[chn]; 
	while(1) { 
		wres = WaitForMultipleObjects( 2, hobj, FALSE, 10000);
		if (wres == WAIT_OBJECT_0) {
			int w = get_width(&G_board[board].m_mode[chn]);
			int h = get_height(&G_board[board].m_mode[chn]);
			if (w > MAX_WIDTH) {
				OutputDebugString("invalid width");
				assert(0); 
				break; 
			}
			if (h > MAX_HEIGHT) {
				OutputDebugString("invalid height");
				assert(0); 
				break;
			}
			if (G_board[board].m_mode[chn].color == COLOR_RGB) {
				unsigned char *pPlane[3];
				unsigned char *pDst;
				image_size2_t img;
				img.height = h;
				img.width = w;
				pPlane[0] = (unsigned char *) G_board[board].m_workbuffer[chn].frame[activebuf].pdata;
				pPlane[2] = (unsigned char *) G_board[board].m_workbuffer[chn].frame[activebuf].pdata + w*h;
				pPlane[1] = (unsigned char *) G_board[board].m_workbuffer[chn].frame[activebuf].pdata + w*h*3/2;
				pDst = (unsigned char *)G_userbuffer[chn].frame[activebuf].pdata;
				// do the conversion
				planar422_to_rgb_mmx(pPlane, pDst, img);

				activebuf++;
				if (activebuf >= G_board[board].m_workbuffer[chn].dwFrames) {
					activebuf = 0;
				}
				ReleaseSemaphore( G_uevents[chn],1, NULL);

			} else if (G_board[board].m_mode[chn].color == COLOR_YUVPK) {
				printf("yuy2 convert\n");
				planar422p_to_yuy2( G_board[board].m_workbuffer[chn].frame[activebuf].pdata,G_userbuffer[chn].frame[activebuf].pdata,
						    w, h);
				ReleaseSemaphore( G_uevents[chn],1, NULL);


			}
		}
		else if (wres == (WAIT_OBJECT_0 + 1)) {
#ifdef _DEBUG
			OutputDebugString("killed\n");
#endif
			break;
		}
	}
	return 0;
}
#endif

static int get_width( MODE2255 *mode) 
{
	int pixelsPerLine = 0;
	if (mode->format == FORMAT_NTSC) {  //NTSC
		switch (mode->scale) {
		case SCALE_4CIFS:
		case SCALE_4CIFSI:
			pixelsPerLine = LINE_SZ_4CIFS_NTSC;
			break;
		case SCALE_2CIFS:
			pixelsPerLine = LINE_SZ_2CIFS_NTSC;
			break;
		case SCALE_1CIFS:
		case SCALE_1CIFSF:
			pixelsPerLine = LINE_SZ_1CIFS_NTSC;
			break;
		default:
			break;
		}
	} else if (mode->format == FORMAT_PAL) {        //PAL, SECAM
		switch (mode->scale) {
		case SCALE_4CIFS:
		case SCALE_4CIFSI:
			pixelsPerLine = LINE_SZ_4CIFS_PAL;
			break;
		case SCALE_2CIFS:
			pixelsPerLine = LINE_SZ_2CIFS_PAL;
			break;
		case SCALE_1CIFS:
		case SCALE_1CIFSF:
			pixelsPerLine = LINE_SZ_1CIFS_PAL;
			break;
		default:
			break;
		}
	}
	return pixelsPerLine;
}

static int get_height( MODE2255 *mode) 
{
	int linesPerFrame = 0;
	if (mode->format == FORMAT_NTSC) {  //NTSC
		switch (mode->scale) {
		case SCALE_4CIFS:
		case SCALE_4CIFSI:
			linesPerFrame = NUM_LINES_4CIFS_NTSC * 2;   //per frame
			break;
		case SCALE_2CIFS:
			linesPerFrame = NUM_LINES_2CIFS_NTSC;
			break;
		case SCALE_1CIFS:
		case SCALE_1CIFSF:
			linesPerFrame = NUM_LINES_1CIFS_NTSC;
			break;
		default:
			break;
		}
	} else if (mode->format == FORMAT_PAL) {        //PAL, SECAM
		switch (mode->scale) {
		case SCALE_4CIFS:
		case SCALE_4CIFSI:
			linesPerFrame = NUM_LINES_4CIFS_PAL * 2;        //per frame
			break;
		case SCALE_2CIFS:
			linesPerFrame = NUM_LINES_2CIFS_PAL;
			break;
		case SCALE_1CIFS:
		case SCALE_1CIFSF:
			linesPerFrame = NUM_LINES_1CIFS_PAL;
			break;
		default:
			break;
		}
	}
	return linesPerFrame;
}

MID2255SPEC UINT32 S2255_get_image_size( MODE2255 *mode) 
{
	int         linesPerFrame = LINE_SZ_DEF;    //total lines per captured unit (frame/field)
	int         pixelsPerLine = NUM_LINES_DEF;  //total pixels per captured line
	UINT32      outImageSize;                   //image size, bytes
	unsigned int  mask_mult;
	linesPerFrame = get_height( mode);
	pixelsPerLine = get_width( mode );

	outImageSize = linesPerFrame * pixelsPerLine;
	switch (mode->color) {
	case COLOR_YUVPL:
	case COLOR_YUVPK:
		outImageSize *= 2;
		break;
	case COLOR_RGB:
		outImageSize *= 3;
		break;
	case COLOR_Y8:
		break;
	}
	return outImageSize;

}

MID2255SPEC int  S2255_DeviceOpen(int board, HDEVICE *phdev )
{
#if defined (OSTYPE_WINDOWS)
#define DEV_NAME_BASE       "\\\\.\\S2255WDM"
#elif defined (OSTYPE_LINUX)
#define DEV_NAME_BASE       "/dev/s2255_"
#endif
	HDEVICE hdev;
	char devname[MAX_PATH];
	ULONG i;
	*phdev = 0;
	sprintf(devname, "%s%01d", DEV_NAME_BASE,board);
	//printf("devname %s\n\n\n", devname);
#if defined(OSTYPE_WINDOWS)
	hdev = CreateFile(devname, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hdev == INVALID_HANDLE_VALUE) {
		return -1;
	}
#elif defined(OSTYPE_LINUX)
	hdev = open( devname, O_RDWR); // | ( BLOCKING_MODE ? 0 : O_NONBLOCK ) );
	if (hdev == 0) {
		return -1;
	}
#endif
	*phdev = hdev;
	G_board[board].hdev = hdev;
	// make work image maximum size(PAL image)
	G_board[board].m_work_image = (unsigned char *) malloc( MAX_WIDTH*MAX_HEIGHT*2);
	for( i=0;i<MAX_CHANNELS;i++) {
		G_board[board].m_acquiring[i] = 0;
	}
	memset( &G_board[board].m_workbuffer[0], 0, MAX_CHANNELS * sizeof(BUFFER));
	return 0;
}

MID2255SPEC int  S2255_DeviceClose(HDEVICE hdev)
{
	int i;
	int board = find_board(hdev);
	if (board == -1) {
		return -1;
	}

	if (G_board[board].m_work_image) {
		free(G_board[board].m_work_image);
	}
	for( i=0;i<MAX_CHANNELS;i++) {
		int j=0;
		G_board[board].m_acquiring[i] = 0;
		if (G_board[board].m_lpbmi[i] ) {
			free( G_board[board].m_lpbmi[i]);
			G_board[board].m_lpbmi[i] = NULL;
		}
		for(j=0;j<SYS_FRAMES;j++) { 
			if (G_board[board].m_workbuffer[i].frame[j].pdata) {
				free(G_board[board].m_workbuffer[i].frame[j].pdata);
				G_board[board].m_workbuffer[i].frame[j].pdata = NULL;
			}
		}
	}
	return CloseHandle( hdev);
}

#if defined (OSTYPE_WINDOWS)
MID2255SPEC int  S2255_AllocBuffer(HDEVICE hdev, int chn, BUFFER *pbuffer, int numbuf)
{
	int i;
	int size;
	int idx = chn-1;
	int board = find_board(hdev);
	if (check_channel(chn) != 0)
		return -1;
	if (board == -1) {
		return -1;
	}

	UNREFERENCED_PARAMETER(hdev);

	size = S2255_get_image_size( &G_board[board].m_mode[idx]);
	pbuffer->dwFrames = numbuf;
	pbuffer->size = size;

	for( i=0; i< numbuf;i++) {
		pbuffer->frame[i].pdata = malloc( size);
	}
	return 0;
}



MID2255SPEC int  S2255_RegBuffer(HDEVICE hdev, int chn, BUFFER *pbuffer)
{
	DWORD retBytes;
	unsigned char *iobuf;
	BOOL res; 
	int i;
#ifdef _DEBUG
	char dbg[50];
#endif
	set_buffer_param_t param;

	int idx = chn-1;
	if (check_channel(chn) != 0)
		return -1;
	if (pbuffer == NULL)
		return STATUS_NO_MEMORY;

#ifdef _DEBUG
	sprintf(dbg, "regbuf %d\n", chn);
	OutputDebugString(dbg);
#endif
	if (G_board[board].m_acquiring[idx] ) {
#ifdef _DEBUG
		sprintf(dbg, "regbuffer: alrdy %d\n", chn);
		OutputDebugString(dbg);
#endif
		return -2;
	}

	assert( sizeof( PVOID) == sizeof( LPBITMAPINFO) );
	iobuf = (unsigned char *) malloc( sizeof(BUFFER) + sizeof(UINT32));
	if (iobuf == NULL) {
		OutputDebugString("Out of memory\n");
		return STATUS_NO_MEMORY;
	}
	// create the global share BMI header( assume for now all images same for all channels )
	if (G_board[board].m_lpbmi[idx] ) {
		free( G_board[board].m_lpbmi[idx]);
		G_board[board].m_lpbmi[idx] = NULL;
	}

	CreateDIBHeader( &G_board[board].m_mode[idx], &G_board[board].m_lpbmi[idx]);
	pbuffer->lpbmi = G_board[board].m_lpbmi[idx];

	if ((G_board[board].m_mode[idx].color != COLOR_RGB) &&  (G_board[board].m_modey[idx].color != COLOR_YUVPK) ) {
		G_userbuffer[idx] = *pbuffer;
		param.idx = idx;
		param.buf = *pbuffer;
	} else {
		// save the user buffer
		G_userbuffer[idx] = *pbuffer;  
		// allocate work buffer and register it with the driver
		{ 
			for(i=0;i<SYS_FRAMES;i++) {
				if (G_board[board].m_workbuffer[idx].frame[i].pdata) {
					free(G_board[board].m_workbuffer[idx].frame[i].pdata);
					G_board[board].m_workbuffer[idx].frame[i].pdata = NULL;
				}
			}
		}
		memset( &G_board[board].m_workbuffer[idx], 0, sizeof(BUFFER));
		S2255_AllocBuffer( hdev, chn, &G_board[board].m_workbuffer[idx], G_userbuffer[idx].dwFrames);
		*(UINT32 *) iobuf = (UINT32) idx;
		param.idx = idx;
		param.buf = G_board[board].m_workbuffer[idx];
	}
	res = pi_ioctl( hdev,  S2255_IOC_SET_BUFFER, &param,sizeof(param), NULL,0, &retBytes, NULL);

	free(iobuf);
	iobuf = NULL;
	if (res) {
		return 0;
	}
	OutputDebugString("call failed Set buffer");
	return -1;
}
#endif


MID2255SPEC int  S2255_SetMode(HDEVICE hdev, int chn, MODE2255 *pmode)
{
	DWORD  retBytes;
	BOOL   res;
	set_mode_param_t param;
	MODE2255I modei;
	//UINT8  pBuf[512];
	int idx = chn-1;
	int board = find_board(hdev);
	if (board == -1) {
		return -2;
	}

	if (check_channel(chn) != 0) {
		return -3;
	}
	copy_mode_to_modei( pmode, &modei);
	// whether DSP needs restarted
	if ((G_board[board].m_mode[idx].scale != modei.scale) || (G_board[board].m_mode[idx].format != modei.format) 
	    || (G_board[board].m_mode[idx].color != modei.color) ) {
		modei.restart = 1;
        }
	else {
		modei.restart = 0;
	}
	// set the mode
	param.idx = idx; 
	modei.usb_block = DEF_USB_BLOCK;
	param.mode = modei;

	res = pi_ioctl( hdev, S2255_IOC_SET_MODE, &param,sizeof(param) , NULL, 0, &retBytes, NULL);

	if (ioctl_pass(res)) { 
		G_board[board].m_mode[idx] = *pmode;
		return 0;
	} else {
		return -1;
	}
}


MID2255SPEC int  S2255_StartAcquire(HDEVICE hdev, int chn, HANDLE *phevent)
{
	BOOL res; 
	reg_event_param_t reg_param;
	start_param_t     start_param;
	// first create call
	HANDLE hevent; 
	DWORD retBytes;
#ifdef _DEBUG
	char dbg[50];
#endif
	int idx = chn-1;
	int board = find_board(hdev);
	if (board == -1) {
		return -1;
	}
	if (check_channel(chn) != 0) {
		return -1;
	}

	if (G_board[board].m_acquiring[idx] ) {
		OutputDebugString("alrdy acquiring\n");
		return -2;
	}
	//printf("start acquire A\n");    
#ifdef _DEBUG
	sprintf(dbg,"startacq %d\n", chn);
	OutputDebugString(dbg);
#endif

#if defined (OSTYPE_WINDOWS)
	// one semaphore per channel
	hevent = CreateSemaphore( NULL,0, G_userbuffer[idx].dwFrames, NULL);
	if (hevent == NULL) {
		OutputDebugString("event create failed\n");
		return STATUS_NO_MEMORY; 
	}
	reg_param.hevent = hevent;
	reg_param.idx = chn-1;
	// register event with driver
	res = pi_ioctl( hdev, S2255_IOC_REG_EVENT, (void *) &reg_param, sizeof( reg_param), NULL,0, &retBytes, NULL);
	if (!ioctl_pass(res)) {
		OutputDebugString("failed to register event\n");
		return -1;
	}
#endif
	// set the driver up for reading from byte
	//printf("start acquire 00\n");    
	start_param.idx = (UINT32) (chn - 1);
	res = pi_ioctl( hdev, S2255_IOC_STARTACQUIRE, (void *) &start_param, sizeof(start_param), NULL, 0, &retBytes, NULL);
	if (!ioctl_pass(res) ) {
		OutputDebugString("failed start acquire\n");
		return -1;
	}

	G_board[board].m_acquiring[idx] = 1;
	//    printf("start acquire 0\n");
#if defined (OSTYPE_WINDOWS)
	if ((G_board[board].m_mode[idx].color == COLOR_RGB) || (G_board[board].m_mode[idx].color == COLOR_YUVPK))
	{
		// start a separate thread
		G_uevents[idx] = CreateSemaphore( NULL, 0, G_userbuffer[idx].dwFrames, NULL);
		G_kill[idx] = CreateEvent(NULL, FALSE, FALSE, NULL);
		G_devents[idx] = hevent;
		G_acq_handle[idx] = CreateThread( NULL, 0, AcquireThread, (LPVOID) (chn-1), 0,(LPDWORD) &G_threadid[idx]);

		*phevent = G_uevents[idx];
	}else {
		// return the event
		G_devents[idx] = hevent;
		*phevent = hevent;
	}
#else
	// Linux: no event returned.  use S2255_check_frame to wait for the frames.
	if (phevent) {
		*phevent = NULL;
	}
#endif
	//printf("start acquire\n");
	return 0;
}


MID2255SPEC int  S2255_StopAcquire(HDEVICE hdev, int chn )
{
	int ret = 0;
	BOOL res; 
	DWORD retBytes;
	stop_param_t stop_param;
	unreg_event_param_t unreg_param;
#ifdef _DEBUG
	char dbg[50];
#endif
	int board = find_board(hdev);
	int idx = chn-1;
	if (check_channel(chn) != 0)
		return -1;
	if (board == -1)
		return -1;
	if (!G_board[board].m_acquiring[idx] ) {
		OutputDebugString("S2255: not alrdy acquiring\n");
		return -2;
	}
#ifdef _DEBUG
	sprintf(dbg, "2255DLL: stop acquire %d\n", chn);
	OutputDebugString(dbg);
#endif
	// stop acquisition in the driver
	stop_param.idx = (UINT32) (chn-1);
	res = pi_ioctl( hdev, S2255_IOC_STOPACQUIRE, (void *) &stop_param, sizeof(stop_param), NULL, 0, &retBytes, NULL);
	if (!ioctl_pass(res) ) {
		OutputDebugString("Stop acquire error\n");
		ret -= 2;
		//return -1;
	}
#ifdef _DEBUG
	OutputDebugString("unreg event\n");
#endif

#if defined (OSTYPE_WINDOWS)
	// unregister the event
	unreg_param.idx = (UINT32) (chn-1);
	res = pi_ioctl( hdev, S2255_IOC_UNREG_EVENT, (void *) &unreg_param, sizeof(unreg_param), NULL, 0, &retBytes, NULL);
	if (!ioctl_pass(res)) {
		printf("failed to unregister event\n");
		ret -=4;
		//return -1;
	}
	if (G_acq_handle[idx]) { 
		DWORD res2;
		SetEvent( G_kill[idx]);
		res2 = WaitForSingleObject( G_acq_handle[idx], 2000);
		if (res2 == WAIT_TIMEOUT) {
			OutputDebugString("failed to kill thread");
		}
		CloseHandle( G_kill[idx]);
		CloseHandle( G_uevents[idx]);

		G_uevents[idx] = NULL;
		G_kill[idx] = NULL;
		G_acq_handle[idx] = NULL;
	}
	if (G_devents[idx]) {
		CloseHandle( G_devents[idx]);
		G_devents[idx] = NULL;
	}
#endif
	G_board[board].m_acquiring[idx] = 0;
	return ret;
}

MID2255SPEC int  S2255_DQBUF(HDEVICE hdev, int chn,int frmnum )
{
	BOOL res; 
	dqbuf_param_t dq_param;
	//    unsigned char tmp[100];
	//    static unsigned int tval=999;
	DWORD retBytes;

	if (check_channel( chn) != 0)
		return -1;

	dq_param.chn = (UINT32) (chn -1);
	dq_param.frm = (UINT32) frmnum;
#if 0 
	// DEBUG only
	// DQ the buffer
	sprintf(tmp, "=============dqbuf %d===================\n", frmnum);
	OutputDebugString(tmp);
	if (tval == frmnum) {
		printf("error\n");
	}
	tval = frmnum;
	if (tval >= 20 ) tval = 0;
#endif

	res = pi_ioctl( hdev, S2255_IOC_DQBUF, (void *)&dq_param, sizeof(dq_param), NULL,0,&retBytes, NULL);
	if (!ioctl_pass(res) ) {
		OutputDebugString("DQBUF error\n");
		return -1;
	}
	return 0;
}

MID2255SPEC int S2255_check_frame_poll_ts( HDEVICE hdev, int chn, int *lastframe, int *size, unsigned int *ts)
{
	BOOL res;
	wait_param_t cmd;
	if (check_channel( chn) != 0) {
		printf("bad channel\n");
		return -1;
	}
	cmd.idx = chn-1;
	cmd.timeout = 0;// doesn't get used for polling, but added anyways
	res = pi_ioctl( hdev, S2255_IOC_WAIT_FRAME_NONBLOCK, &cmd,0,0,0,0,0);
	if (!ioctl_pass(res) ) {
		return -1;
	}
	if (lastframe) {
		*lastframe = cmd.last_frame;
	}
	if (size)
		*size = cmd.last_size;

	if (ts)
		*ts = cmd.last_ts;

	if (cmd.last_frame == -1) {
		return -1;
	}
	return 0;
}

MID2255SPEC int S2255_check_frame_poll( HDEVICE hdev, int chn, int *lastframe, int *size)
{
	return S2255_check_frame_poll_ts(hdev, chn, lastframe, size, NULL);
}


MID2255SPEC int S2255_check_frame_block_ts( HDEVICE hdev, int chn, int timeout, int *lastframe, int *size, unsigned int *ts)
{
	BOOL res;
	wait_param_t cmd;
	if (check_channel( chn) != 0) {
		printf("bad channel\n");
		return -1;
	}
	memset(&cmd, 0, sizeof(cmd));
	cmd.idx = chn-1;
	cmd.timeout = timeout;

	res = pi_ioctl( hdev,S2255_IOC_WAIT_FRAME_BLOCK, &cmd,0,0,0,0,0);
	if (!ioctl_pass(res) ) {
		return -1;
	}
	if (lastframe) {
		*lastframe = cmd.last_frame;
		//        printf("last frame %d\n", *plastframe);
	}
	if (size)
		*size = cmd.last_size;

	if (ts)
		*ts = cmd.last_ts;

	if (cmd.last_frame == -1) {
		return -1;
	}

	return 0;
}

MID2255SPEC int S2255_check_frame_block( HDEVICE hdev, int chn, int timeout, int *lastframe, int *size)
{
	return S2255_check_frame_block_ts(hdev, chn, timeout, lastframe, size, NULL);
}

MID2255SPEC int S2255_get_buffer( HDEVICE hdev, int chn, int frmnum, char *pbuf, int size)
{
	BOOL res;
	getbuf_param_t cmd;
	if (check_channel( chn) != 0) {
		printf("bad channel\n");
		return -1;
	}
	cmd.idx = chn-1;
	cmd.frm_num = frmnum;
	cmd.pbuf = pbuf;
	cmd.size = size;
	//printf("calling ioctl %d, size %d\n", chn, size);
	res = pi_ioctl( hdev,S2255_IOC_COPY_BUFFER, &cmd,0,0,0,0,0);
	if (!ioctl_pass(res) ) {
		return -1;
	}
	return 0;
}



MID2255SPEC int CreateDIBHeader( MODE2255 *pmode, LPBITMAPINFO * plpbmi)
{
	LPBITMAPINFO ptrHeader;
	DWORD headersize;
	int i;
	/* Create DIB */
	if (pmode->color == COLOR_Y8) {
		headersize = sizeof (BITMAPINFOHEADER) + 256 * sizeof (RGBQUAD);
	}
	else {
		headersize = sizeof (BITMAPINFOHEADER);
	}
	/* Allocate memory for DIB info */
	ptrHeader = (LPBITMAPINFO) malloc (headersize);

	if (ptrHeader != NULL) {                    //if memory allocated;
		ptrHeader -> bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
		ptrHeader -> bmiHeader.biWidth = get_width( pmode); //640; //pmode -> advanced.xscale.size;
		ptrHeader -> bmiHeader.biHeight = get_height(pmode); //480; // pmode -> advanced.yscale.size;
		ptrHeader -> bmiHeader.biPlanes = 1;
		ptrHeader -> bmiHeader.biCompression = 0;
		ptrHeader -> bmiHeader.biSizeImage = 0;         //no need if no compression;
		ptrHeader -> bmiHeader.biXPelsPerMeter = 0;
		ptrHeader -> bmiHeader.biYPelsPerMeter = 0;

		ptrHeader -> bmiHeader.biBitCount = 24;
		ptrHeader -> bmiHeader.biClrUsed = 0;
		ptrHeader -> bmiHeader.biClrImportant = 0;

		if (pmode->color == COLOR_Y8) {
			/*create grey scale color table */
			for (i = 0; i < 256; i++) {
				ptrHeader->bmiColors[i].rgbBlue = i & 0xff; 
				ptrHeader->bmiColors[i].rgbGreen = i & 0xff; 
				ptrHeader->bmiColors[i].rgbRed = i & 0xff;   
			}

			ptrHeader -> bmiHeader.biBitCount = 8;
			ptrHeader -> bmiHeader.biClrUsed = 256;
			ptrHeader -> bmiHeader.biClrImportant = 0;
		}
		*plpbmi = ptrHeader;
		return 0;
	}
	else {
		return -1;
	}
}

MID2255SPEC int S2255_GetSN(HDEVICE hdev, SN2255 *sn )
{
		
	UINT8 outbuf[32];
	BOOL res;
	DWORD retbytes;
	UINT16 reg = 8 * 1024 -16;

	get_sn_param_t cmd;

	cmd.addr = 0xA2;
	cmd.subaddr = 0x1FF0;
	cmd.data = outbuf;
	cmd.datalen = 16;	
	
	res = pi_ioctl( hdev, SX2255_IOC_READ_I2C_BURST, &cmd,0,0,0,0,0 );	
	
	if (!ioctl_pass(res) ) {
		return res;		
	}
	
	// Check for marker code
	if (*(unsigned int *)outbuf != 0xddccbbaa) {
		return -15;
	}

	sn->serial_number = (outbuf[12] << 24) + (outbuf[13] << 16) + (outbuf[14] << 8) + outbuf[15];
	
	sn->month = outbuf[9];
	sn->day = outbuf[10];
	sn->year = outbuf[11];

	return 0;
}


MID2255SPEC int S2255_GetFirmware(HDEVICE hdev, UINT32 *usb_fw, UINT32 *dsp_fw)
{
	BOOL res;
	DWORD retbytes;
	get_firmware_t cmd;
	if (usb_fw == NULL)
		return -1;
	if (dsp_fw == NULL)
		return -1;
	res = pi_ioctl( hdev, S2255_IOC_GET_FIRMWARE, &cmd,0,0,0,0,0 );	
	if (!ioctl_pass(res) ) {
		return res;		
	}
	*usb_fw = cmd.usb_fw;
	*dsp_fw = cmd.dsp_fw;
	return 0;
}


MID2255SPEC int  S2255_query_buf(HDEVICE hdev, int chn, int frmnum)
{
    querybuf_t cmd;
	BOOL res;
	DWORD retBytes=0;
	int idx = chn-1;
	if (check_channel(chn) != 0) {
		return -1;
    }
    cmd.chn = idx;
    cmd.frm = frmnum;
	res = pi_ioctl(hdev, S2255_IOC_QUERYBUF, &cmd, sizeof(cmd) , &cmd, sizeof(cmd), &retBytes, NULL);
	if (ioctl_pass(res) ) {
		return cmd.state;
	} else {
        return -1;
    }
}

MID2255SPEC int  S2255_query_buf_ext( HDEVICE hdev, int chn, int frmnum, int *ts)
{
    querybuf_ext_t cmd;
	BOOL res;
	DWORD retBytes=0;
	int idx = chn-1;
	if (check_channel(chn) != 0)
		return -1;
    cmd.chn = idx;
    cmd.frm = frmnum;
    cmd.ts = 0;
	res = pi_ioctl(hdev, S2255_IOC_QUERYBUF_EXT, &cmd, sizeof(cmd) , &cmd, sizeof(cmd), &retBytes, NULL);
	if (ioctl_pass(res) ) {
        *ts = cmd.ts;
		return cmd.state;
	} else
        return -1;
}


MID2255SPEC int  S2255_query_buf_all(HDEVICE hdev, int chn, int state[SYS_FRAMES], int ts[SYS_FRAMES])
{
    querybuf_all_t cmd;
	BOOL res;
	DWORD retBytes=0;
	int idx = chn-1;
	if (check_channel(chn) != 0) {
		return -1;
    }
    cmd.chn = idx;
	res = pi_ioctl(hdev, S2255_IOC_QUERYBUF_ALL, &cmd, sizeof(cmd) , &cmd, sizeof(cmd), &retBytes, NULL);
    memcpy(state, cmd.state, SYS_FRAMES * sizeof(int));
    memcpy(ts, cmd.ts, SYS_FRAMES * sizeof(int));
	if (ioctl_pass(res) ) {
		return 0;
	} else {
        return -1;
    }
}

#ifdef S2255F_INTERNAL // internal debug functions
MID2255SPEC int  S2255_get_busy_buffers( HDEVICE hdev, int chn)
{
	unsigned char buf[4];
	BOOL res;
	DWORD retBytes=0;
	UINT32   busy_buffers;
	if (check_channel(chn) != 0)
		return -1;

	*(UINT32 *) buf = chn-1;
	res = pi_ioctl( hdev, S2255_IOC_GET_BUSY_BUFFERS, buf,4 , &busy_buffers, sizeof(UINT32), &retBytes, NULL);
	if (res) {
		return busy_buffers;
	}
	else return -1;
}

MID2255SPEC int  S2255_get_frame_count( HDEVICE hdev)
{
	unsigned char buf[4];
	BOOL res;
	DWORD retBytes=0;
	UINT32   retval;
	*(UINT32 *) buf = 0;
	res = pi_ioctl( hdev, S2255_IOC_GET_FRAME_COUNT, buf,4 , &retval, 4, &retBytes, NULL);
	if (res) {
		return retval;
	}
	else return -1;
}
#endif


MID2255SPEC int  S2255_get_vid_status(HDEVICE hdev, int chn, UINT32 *pStatus)
{
        get_vidstatus_t param;
	BOOL res;
	DWORD retBytes=0;
	UINT32   retval;
	int idx = chn-1;
	param.idx = idx;
	res = pi_ioctl( hdev, S2255_IOC_GET_STATUS, &param, sizeof(param) , &param, sizeof(param), &retBytes, NULL);
	if(ioctl_pass(res)) {
		*pStatus = param.status;
		printf("pass %d\n", param.status);
		return 0;
	} 
	return -1;
}

