#include "s2255_i.h"
#ifdef DRIVER_BUILD
#include "s2255core.h"
#include "s2255mod.h"
#include "2255vx.h"
#include "debug.h"
#include "s2255_i.h"
#include "size.h"

#endif

#ifndef DEF_USB_BLOCK
#define DEF_USB_BLOCK (16*1024)
#endif
#define LINE_SZ_4CIFS_NTSC  640
#define LINE_SZ_2CIFS_NTSC  640
#define LINE_SZ_1CIFS_NTSC  320

#define LINE_SZ_4CIFS_PAL  704 //720//768
#define LINE_SZ_2CIFS_PAL  704 //20//768
#define LINE_SZ_1CIFS_PAL  352 //360//384

#define NUM_LINES_4CIFS_NTSC 240
#define NUM_LINES_2CIFS_NTSC 240
#define NUM_LINES_1CIFS_NTSC 240

#define NUM_LINES_4CIFS_PAL  288 // 288
#define NUM_LINES_2CIFS_PAL  288
#define NUM_LINES_1CIFS_PAL  288

#define LINE_SZ_DEF				640
#define NUM_LINES_DEF			240			//this is per field!

#define USB_READ_SIZE		0x00001000		
//depends on USB_READ_SIZE
#define MASK_MULT			0xfffff000
#define PREFIX_SIZE			512				//frame prefix size (sent once every frame)

#ifdef WIN32
#pragma LOCKEDCODE
#endif


UINT32 get_transfer_size( MODE2255I *mode) 
{
    int			linesPerFrame = LINE_SZ_DEF;	//total lines per captured unit (frame/field)
    int			pixelsPerLine = NUM_LINES_DEF;	//total pixels per captured line
    UINT32      outImageSize;					//image size, bytes
    UINT32      usbInSize;
    unsigned int  mask_mult;

    if (mode == 0) {
        //dbgprint("get size err!\n");
        return 0;
    }
    //printk("mode %p", mode);
    
    if (mode->format == FORMAT_NTSC) {	//NTSC
        switch (mode->scale) {
        case SCALE_4CIFS:
        case SCALE_4CIFSI:
            linesPerFrame = NUM_LINES_4CIFS_NTSC * 2;	//per frame
            pixelsPerLine = LINE_SZ_4CIFS_NTSC;
            break;
        case SCALE_2CIFS:
            linesPerFrame = NUM_LINES_2CIFS_NTSC;
            pixelsPerLine = LINE_SZ_2CIFS_NTSC;
            break;
        case SCALE_1CIFS:
        case SCALE_1CIFSF:
            linesPerFrame = NUM_LINES_1CIFS_NTSC;
            pixelsPerLine = LINE_SZ_1CIFS_NTSC;
            break;
        default:
            break;
        }
    } else if (mode->format == FORMAT_PAL) {		//PAL, SECAM
        switch (mode->scale) {
        case SCALE_4CIFS:
        case SCALE_4CIFSI:
            linesPerFrame = NUM_LINES_4CIFS_PAL * 2;		//per frame
            pixelsPerLine = LINE_SZ_4CIFS_PAL;
            break;
        case SCALE_2CIFS:
            linesPerFrame = NUM_LINES_2CIFS_PAL;
            pixelsPerLine = LINE_SZ_2CIFS_PAL;
            break;
        case SCALE_1CIFS:
        case SCALE_1CIFSF:
            linesPerFrame = NUM_LINES_1CIFS_PAL;
            pixelsPerLine = LINE_SZ_1CIFS_PAL;
            break;
        default:
            break;
        }
    }
    outImageSize = linesPerFrame * pixelsPerLine;
    if (mode->color != COLOR_Y8) {
        outImageSize *= 2;	//2 bytes/pixel if not monochrome
    }

    //total bytes to send including prefix and 4K padding; must be a multiple of USB_READ_SIZE
    usbInSize = outImageSize + PREFIX_SIZE;	//always send prefix
    mask_mult = 0xffffffffUL - DEF_USB_BLOCK + 1 ;
    if (usbInSize & ~mask_mult) {				//if size not a multiple of USB_READ_SIZE
        usbInSize = (usbInSize & mask_mult) + (DEF_USB_BLOCK); //mode->usb_block; //USB_READ_SIZE;
    }
//    KdPrint(("usb block %d\n", mode->usb_block));
    return usbInSize;
}
